// CONJGRAD.H : a general-purpose conjugate-gradient optimizer class.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef CONJGRAD_H
#define CONJGRAD_H

class conjugate_gradient;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "define.h"
#include "typedef.h"

#include <vector>

////////////////////////////////////////////////////////////////////////////////////////////////////

struct cgvar
{
	f64 * ref1;	// ref to the variable
	f64 * ref2;	// ref to the derivative
	
	f64 data1;
	f64 data2;
};

class conjugate_gradient
{
	public:
	
	enum ls_mode			// line-search modes
	{
		Simple = 0,		// simple and safe testing method
		Newton2An = 1,		// f'/f''-newton's method, half numerical
		Newton2Num = 2,		// f'/f''-newton's method, full numerical
		Newton1Num = 3		// f/f'-newton's method, full numerical
	};
	
	protected:
	
	vector<cgvar> cgvar_vector;
	f64 newip; f64 oldip; f64 beta;
	
	public:
	
	i32s step; i32s reset;
	f64 defstp; f64 delta;
	
	f64 optstp; f64 optval;
	
	public:
	
	conjugate_gradient(i32s, f64, f64 = +1.0e-08);
	virtual ~conjugate_gradient(void);
	
	void AddVar(f64 *, f64 *);
	
	void TakeCGStep(ls_mode);
	void InitLineSearch(f64);
	
	virtual f64 GetValue(void) = 0;
	virtual f64 GetGradient(void);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// CONJGRAD_H

// eof
