// MM1MDL.H : data storage for "all atoms"-models (molecular mechanics).

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef MM1MDL_H
#define MM1MDL_H

class mm1_mdl;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "mdlbase.h"
#include "mm1util.h"

#include <list>
#include <vector>

#define NOT_FOUND 0x7fffffff	// numeric_limits<i32s>::max()?!?!?!

////////////////////////////////////////////////////////////////////////////////////////////////////

struct mm1_ent_import1
{
	signed char chn; i32s res;
	mm1_atom * ref;
};

struct mm1_ent_import2
{
	signed char chn; i32s res;
	char res_name[5]; char atm_name[5];
	fGL crd[3]; mm1_atom * ref;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class mm1_mdl : public bs_mdl_extended
{
	public:
	
	static mm1_sequencebuilder amino_builder;
	static mm1_sequencebuilder nucleic_builder;
	
	i32s nmol;
	vector<mm1_chn_info *> * ref_rci_v;
	
	list<mm1_atom> atom_list;
	list<mm1_bond> bond_list;
	
	f64 box_hdim[3];
	f64 box_fdim[3];
	
	public:
	
	mm1_mdl(ostream *);
	~mm1_mdl(void);
	
	const char * GetProjectFileNameExtension(void);		// virtual
	
	void PushCRDSets(i32u);			// virtual
	void PopCRDSets(i32u);			// virtual
	
	void CopyCRDSet(i32u, i32u);		// virtual
	void SwapCRDSets(i32u, i32u);		// virtual
	
	void CenterCRDSet(i32u);		// virtual
	void ReserveCRDSets(i32u);		// virtual
	
	// import method for hyperchem HIN-files:
	
	bool ImportHIN(const char *, bool = true);
	
	// import method for ENT-files from the brookhaven PDB:
	
	void ImportENT(const char *, signed char);
	i32s ImportENT_sub1(vector<mm1_ent_import2> &, i32s *, const char *, bool);
	void ImportENT_sub2(vector<mm1_ent_import2> &, i32s *, const char *, const char *, char);
	
	// methods for adding new atoms and bonds:
	
	void AddAtom(mm1_atom &);
	void RemoveAtom(iter_mm1al);
	
	void AddBond(mm1_bond &);
	void RemoveBond(iter_mm1bl);
	
	void ReadStream(istream &);
	void WriteStream(ostream &);
	
	void AddHydrogens(void);
	void RemoveHydrogens(void);
	
	void GatherGroups(void);
	void GetRange(i32s, i32s, iter_mm1al *);
	void GetRange(i32s, iter_mm1al *, i32s, iter_mm1al *);
	void InvalidateGroups(void);
	
	i32s FindPath(mm1_atom *, mm1_atom *, i32s, i32s, i32s = 0);
	bool FindRing(mm1_atom *, mm1_atom *, signed char *, i32s, i32s, i32s = 0);
	
	void UpdateIndex(void);
	
	private:
	
	void GatherAtoms(mm1_atom *, i32s);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// MM1MDL_H

// eof
