// QM2ENG.H : a base class for qm2-engine classes.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef QM2ENG_H
#define QM2ENG_H

class qm2_eng;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "engbase.h"
#include "qm2mdl.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

// qm2-models can have different basis functions and other such different details, so it is not
// as straightforward to draw planes/surfaces for those than to molecular mechanics models.

// therefore we say that qm2-engine classes must provide us the values we need for planes/surfaces.

void CopyCRD(qm2_mdl *, qm2_eng *, i32u);
void CopyCRD(qm2_eng *, qm2_mdl *, i32u);

class qm2_eng : public bs_eng
{
	private:
	
	qm2_mdl & model;
	
	protected:
	
	i32u natm;
	
	public:
	
	f64 energy;
	
	public:
	
	qm2_eng(qm2_mdl &);
	virtual ~qm2_eng(void);
	
	qm2_mdl * GetModel(void) { return & model; }
	
	// geometry changes?!?!?!
	
	virtual const f64 * GetCRD(i32u) = 0;
	virtual void PutCRD(i32u, const f64 *) = 0;
	
	// these are for plotting purposes...
	
	virtual fGL GetESP(fGL *, fGL *) = 0;
	virtual fGL GetElDens(fGL *, fGL *) = 0;
	
	virtual fGL GetOrbital(fGL *, fGL *) = 0;
	virtual fGL GetOrbDens(fGL *, fGL *) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// QM2ENG_H

// eof
