// QM2ENG1.H : the MPQC front-end is implemented here.

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef QM2ENG1_H
#define QM2ENG1_H

#ifdef ENABLE_MPQC

class qm2_eng_mpqc;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "qm2eng.h"
#include "qm2mdl.h"

#include <fstream>

#include "util/keyval/keyval.h"			// KeyVal

#include "util/group/message.h"			// MessageGrp
#include "util/group/thread.h"			// ThreadGrp

#include "chemistry/molecule/energy.h"		// MolecularEnergy
#include "chemistry/qc/wfn/obwfn.h"		// OneBodyWavefunction
#include "chemistry/qc/wfn/wfn.h"		// Wavefunction

#include "math/scmat/abstract.h"		// SCMatrix

////////////////////////////////////////////////////////////////////////////////////////////////////

// strategy :
// initialize using a standard MPQC input file -> we will have also an input file generator.
// for each different situation (closed shell, open shell, whatever) we will have a different class.
// this is only for closed shell cases -> split this to a base class/derived classes in future!!!!!
// for example, open shell cases might have separate matrices for alpha and beta orbitals etc...

class qm2_eng_mpqc : public qm2_eng
{
	protected:
	
	static i32u ifile_name_counter;
	
	RefMessageGrp grp;
	RefThreadGrp thread;
	
	RefParsedKeyVal parsedkv;
	RefKeyVal keyval;
	
	RefMolecularEnergy mole;
	
	RefOneBodyWavefunction obwfn;		// for Hartree-Fock orbitals...
	
	RefWavefunction wfn;			// for natural orbitals...
	RefSCMatrix natorb;			// for natural orbitals...
	
	public:
	
	qm2_eng_mpqc(qm2_mdl &);
	~qm2_eng_mpqc(void);
	
	void GetInputFileName(char *, i32u);
	void WriteInputFile(ofstream &);
	
	const f64 * GetCRD(i32u);		// virtual
	void PutCRD(i32u, const f64 *);		// virtual
	
	void Compute(i32s);		// virtual
	void Check(i32s);		// virtual
	
	fGL GetESP(fGL *, fGL *);		// virtual
	fGL GetElDens(fGL *, fGL *);		// virtual
	
	fGL GetOrbital(fGL *, fGL *);		// virtual
	fGL GetOrbDens(fGL *, fGL *);		// virtual
	
	static fGL getESP(fGL * pp, qm2_eng * ref, fGL * dd) { qm2_eng_mpqc * ref2 = (qm2_eng_mpqc *) ref; return ref2->GetESP(pp, dd); }
	static fGL getElDens(fGL * pp, qm2_eng * ref, fGL * dd) { qm2_eng_mpqc * ref2 = (qm2_eng_mpqc *) ref; return ref2->GetElDens(pp, dd); }
	
	static fGL getOrbital(fGL * pp, qm2_eng * ref, fGL * dd) { qm2_eng_mpqc * ref2 = (qm2_eng_mpqc *) ref; return ref2->GetOrbital(pp, dd); }
	static fGL getOrbDens(fGL * pp, qm2_eng * ref, fGL * dd) { qm2_eng_mpqc * ref2 = (qm2_eng_mpqc *) ref; return ref2->GetOrbDens(pp, dd); }
};

////////////////////////////////////////////////////////////////////////////////////////////////////

// strategy :
// create the MPQC-engine inside this class and append all optimizer-stuff to the input file.
// so it seems we can't simply follow the MM idea of engine -> algorithms here !??!?!

// on the other hand, we could make exactly similar algorithms here like in MM section,
// but those will less efficient if there is a "native" counterpart.

class qm2_mpqc_optimizer	// base class?!?!?!?
{
	protected:
	
	qm2_eng_mpqc * eng;
	
	public:
	
	qm2_mpqc_optimizer(qm2_mdl &);
	~qm2_mpqc_optimizer(void);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_MPQC
#endif	// QM2ENG1_H

// eof
