// QM2MDL.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "qm2mdl.h"

#include <fstream>

////////////////////////////////////////////////////////////////////////////////////////////////////

qm2_atom::qm2_atom(void)
{
}

qm2_atom::qm2_atom(element p1, fGL * p2, i32u p3)
{
	el = p1;
	
	fGL_a3 data = { p2[0], p2[1], p2[2] };
	for (i32u n1 = 0;n1 < p3;n1++) crd_vector.push_back(data);
}

qm2_atom::qm2_atom(const qm2_atom & p1)
{
	el = p1.el;
	crd_vector = p1.crd_vector;
}

qm2_atom::~qm2_atom(void)
{
}

// for qm2_atoms, equality is tested using pointers -> you can find the
// iterator (using STL's find-function) if the pointer is known.

bool qm2_atom::operator==(const qm2_atom & p1) const
{
	return (this == (& p1));
}

////////////////////////////////////////////////////////////////////////////////////////////////////

qm2_mdl::qm2_mdl(ostream * p1) : bs_mdl_extended(p1), bs_mdl_simple(p1)
{
	current_eng = NULL;
	current_orbital = 0;
}

qm2_mdl::~qm2_mdl(void)
{
	if (current_eng != NULL) delete current_eng;
}

const char * qm2_mdl::GetProjectFileNameExtension(void)
{
	static const char ext[] = "qm2gp";
	return ext;
}

void qm2_mdl::DiscardCurrentEng(void)
{
	if (current_eng != NULL)
	{
		delete current_eng;
		current_eng = NULL;
	}
}

// this stuff is pretty much stolen from the mm1_mdl...
// this stuff is pretty much stolen from the mm1_mdl...
// this stuff is pretty much stolen from the mm1_mdl...

void qm2_mdl::PushCRDSets(i32u p1)
{
//	cs_vector.resize(cs_vector.size() + p1);			// more convenient way?!?!?!
//	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back();		// more crash-proof way?!?!?!?
	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());	// or even this way?!?!?!?
	
	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
	for (iter_qm2al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32u n1 = 0;n1 < p1;n1++)
		{
			(* it1).crd_vector.push_back(newcrd);
		}
	}
}

void qm2_mdl::PopCRDSets(i32u)
{
}

void qm2_mdl::CopyCRDSet(i32u, i32u)
{
}

void qm2_mdl::SwapCRDSets(i32u, i32u)
{
}

void qm2_mdl::CenterCRDSet(i32u p1)
{
	fGL sum[3] = { 0.0, 0.0, 0.0 };
	for (iter_qm2al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32s n1 = 0;n1 < 3;n1++) sum[n1] += (* it1).crd_vector[p1][n1];
	}
	
	for (iter_qm2al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32s n1 = 0;n1 < 3;n1++)
		{
			fGL tmp1 = sum[n1] / (f64) atom_list.size();
			(* it1).crd_vector[p1][n1] -= tmp1;
		}
	}
}

void qm2_mdl::ReserveCRDSets(i32u)
{
}

void qm2_mdl::AddAtom(qm2_atom & p1)
{
	DiscardCurrentEng();
	atom_list.push_back(p1);
}

void qm2_mdl::RemoveAtom(iter_qm2al it1)
{
	DiscardCurrentEng();
	atom_list.erase(it1);
}

void qm2_mdl::ImportMM1(const char * fn)
{
        ifstream ifile(fn, ios::in);
        
        i32s tmp2[2]; fGL tmp4[3];
	while (ifile.peek() != 'E')
	{
		ifile >> tmp2[0] >> tmp2[1];
		ifile >> tmp4[0] >> tmp4[1] >> tmp4[2] >> tmp2[0];
		
		element el = element(tmp2[1]);
		qm2_atom newatom(el, tmp4, cs_vector.size());
		AddAtom(newatom);
		
		ifile.getline(buffer, sizeof(buffer));
	}
	
	ifile.close();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
