// UTILITY.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "utility.h"
#include "define.h"

#include <iostream>

////////////////////////////////////////////////////////////////////////////////////////////////////

// this function can be used to modify a 3D-vector object using a 4x4 transformation matrix.
// will first make a [1*4]x[4x4]-type matrix product and after that bring the result back to 3D.
// for more complete documentation look/search for "homogeneous coordinates"...

void TransformVector(v3d<fGL> & p1, const fGL * p2)
{
	i32s n1; i32s n2; fGL pv[4]; pv[3] = 1.0;
	for (n1 = 0;n1 < 3;n1++) pv[n1] = p1.data[n1];
	
	fGL rv[4] = { 0.0, 0.0, 0.0, 0.0 };
	for (n1 = 0;n1 < 4;n1++) for (n2 = 0;n2 < 4;n2++) rv[n1] += pv[n2] * p2[n2 * 4 + n1];
	for (n1 = 0;n1 < 3;n1++) p1.data[n1] = rv[n1] / rv[3];
}

////////////////////////////////////////////////////////////////////////////////////////////////////

element element::current_element = element("C");

const char * element::string[ELEMENT_SYMBOLS] =
{
	"hydrogen", "helium", "lithium", "beryllium", "boron",
	"carbon", "nitrogen", "oxygen", "fluorine", "neon",				// 10
	"sodium", "magnesium", "aluminium", "silicon", "phosphorus",
	"sulphur", "chlorine", "argon", "potassium", "calcium",				// 20
	"scandium", "titanium", "vanadium", "chromium", "manganese",
	"iron", "cobalt", "nickel", "copper", "zinc",					// 30
	"gallium", "germanium", "arsenic", "selenium", "bromine",
	"krypton", "rubidium", "strontium", "yttrium", "zirconium",			// 40
	"niobium", "molybdenum", "technetium", "ruthenium", "rhodium",
	"palladium", "silver", "cadmium", "indium", "tin",				// 50
	"antimony", "tellurium", "iodine", "xenon", "cesium",
	"barium", "lanthanum", "cerium", "praseodymium", "neodymium",			// 60
	"promethium", "samarium", "europium", "gadolinium", "terbium",
	"dysprosium", "holmium", "erbium", "thulium", "ytterbium",			// 70
	"lutetium", "hafnium", "tantalum", "tungsten", "rhenium",
	"osmium", "iridium", "platinum", "gold", "mercury",				// 80
	"thallium", "lead", "bismuth", "polonium", "astatine",
	"radon", "francium", " radium", "actinium", "thorium",				// 90
	"protactinium", "uranium", "neptunium", "plutionium", "americium",
	"curium", "berkelium", "californium", "einsteinium", "fermium",			// 100
	"mendelevium", "nobelium", "lawrencium", "unnilquadium", "unnilpentium"
};

const char element::symbol[ELEMENT_SYMBOLS][3] =
{
	"H", "He", "Li", "Be", "B", "C", "N", "O", "F", "Ne",
	"Na", "Mg", "Al", "Si", "P", "S", "Cl", "Ar", "K", "Ca",		// 20
	"Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn",
	"Ga", "Ge", "As", "Se", "Br", "Kr", "Rb", "Sr", "Y", "Zr",		// 40
	"Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd", "In", "Sn",
	"Sb", "Te", "I", "Xe", "Cs", "Ba", "La", "Ce", "Pr", "Nd",		// 60
	"Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", "Yb",
	"Lu", "Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg",		// 80
	"Tl", "Pb", "Bi", "Po", "At", "Rn", "Fr", "Ra", "Ac", "Th",
	"Pa", "U", "Np", "Pu", "Am", "Cm", "Bk", "Cf", "Es", "Fm",		// 100
	"Md", "No", "Lr", "Rf", "Ha"
};

const fGL element::color[ELEMENT_SYMBOLS][3] =
{
	{ 0.7, 0.7, 0.7 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 0.0, 1.0, 0.0 },	// 6
	{ 0.0, 0.0, 1.0 }, { 1.0, 0.0, 0.0 }, { 0.5, 0.2, 0.7 },
	{ 0.7, 0.7, 0.5 }, { 0.7, 0.7, 0.7 }, { 1.0, 1.0, 1.0 },	// 12
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },
	{ 0.0, 1.0, 0.0 }, { 0.0, 0.0, 1.0 }, { 1.0, 0.0, 0.0 },	// 18
	{ 0.5, 0.2, 0.7 }, { 0.7, 0.7, 0.5 }, { 0.7, 0.7, 0.7 },
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },	// 24
	{ 1.0, 1.0, 1.0 }, { 0.0, 1.0, 0.0 }, { 0.0, 0.0, 1.0 },
	{ 1.0, 0.0, 0.0 }, { 0.5, 0.2, 0.7 }, { 0.7, 0.7, 0.5 },	// 30
	{ 0.7, 0.7, 0.7 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 0.0, 1.0, 0.0 },	// 36
	{ 0.0, 0.0, 1.0 }, { 1.0, 0.0, 0.0 }, { 0.5, 0.2, 0.7 },
	{ 0.7, 0.7, 0.5 }, { 0.7, 0.7, 0.7 }, { 1.0, 1.0, 1.0 },	// 42
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },
	{ 0.0, 1.0, 0.0 }, { 0.0, 0.0, 1.0 }, { 1.0, 0.0, 0.0 },	// 48
	{ 0.5, 0.2, 0.7 }, { 0.7, 0.7, 0.5 }, { 0.7, 0.7, 0.7 },
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },	// 54
	{ 1.0, 1.0, 1.0 }, { 0.0, 1.0, 0.0 }, { 0.0, 0.0, 1.0 },
	{ 1.0, 0.0, 0.0 }, { 0.5, 0.2, 0.7 }, { 0.7, 0.7, 0.5 },	// 60
	{ 0.7, 0.7, 0.7 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 0.0, 1.0, 0.0 },	// 66
	{ 0.0, 0.0, 1.0 }, { 1.0, 0.0, 0.0 }, { 0.5, 0.2, 0.7 },
	{ 0.7, 0.7, 0.5 }, { 0.7, 0.7, 0.7 }, { 1.0, 1.0, 1.0 },	// 72
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },
	{ 0.0, 1.0, 0.0 }, { 0.0, 0.0, 1.0 }, { 1.0, 0.0, 0.0 },	// 78
	{ 0.5, 0.2, 0.7 }, { 0.7, 0.7, 0.5 }, { 0.7, 0.7, 0.7 },
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },	// 84
	{ 1.0, 1.0, 1.0 }, { 0.0, 1.0, 0.0 }, { 0.0, 0.0, 1.0 },
	{ 1.0, 0.0, 0.0 }, { 0.5, 0.2, 0.7 }, { 0.7, 0.7, 0.5 },	// 90
	{ 0.7, 0.7, 0.7 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 },
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 0.0, 1.0, 0.0 },	// 96
	{ 0.0, 0.0, 1.0 }, { 1.0, 0.0, 0.0 }, { 0.5, 0.2, 0.7 },
	{ 0.7, 0.7, 0.5 }, { 0.7, 0.7, 0.7 }, { 1.0, 1.0, 1.0 },	// 102
	{ 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }, { 1.0, 1.0, 1.0 }
};


const fGL element::mass[ELEMENT_SYMBOLS] =
{
	1.008, 4.003, 6.941, 9.012, 10.81, 12.01, 14.01, 16.00, 19.00, 20.12,
	22.99, 24.31, 26.98, 28.09, 30.97, 32.06, 35.45, 39.95, 39.10, 40.08,	// 20
	
	// what is the argon mass??? complete this table...
	
	0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
	0.15, 0.15, 0.15, 0.15, 0.19, 0.15, 0.15, 0.15, 0.15, 0.15,	// 40
	0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
	0.15, 0.15, 0.15, 0.15, 0.19, 0.15, 0.15, 0.15, 0.15, 0.15,	// 60
	0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
	0.15, 0.15, 0.15, 0.15, 0.19, 0.15, 0.15, 0.15, 0.15, 0.15,	// 80
	0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
	0.15, 0.15, 0.15, 0.15, 0.19, 0.15, 0.15, 0.15, 0.15, 0.15,	// 100
	0.15, 0.15, 0.15, 0.15, 0.15
};

// check these...
// check these...
// check these...

const fGL element::vdwr[ELEMENT_SYMBOLS] =
{
	0.11, 0.12, 0.15, 0.15, 0.15, 0.15, 0.15, 0.14, 0.14, 0.20,
	0.15, 0.15, 0.15, 0.15, 0.19, 0.19, 0.15, 0.15, 0.15, 0.15,	// 20
	0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
	0.15, 0.15, 0.15, 0.15, 0.19, 0.15, 0.15, 0.15, 0.15, 0.15,	// 40
	0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
	0.15, 0.15, 0.15, 0.15, 0.19, 0.15, 0.15, 0.15, 0.15, 0.15,	// 60
	0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
	0.15, 0.15, 0.15, 0.15, 0.19, 0.15, 0.15, 0.15, 0.15, 0.15,	// 80
	0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15, 0.15,
	0.15, 0.15, 0.15, 0.15, 0.19, 0.15, 0.15, 0.15, 0.15, 0.15,	// 100
	0.15, 0.15, 0.15, 0.15, 0.15
};

element::element(void)
{
	atomic_number = NOT_DEFINED;
}

// unknown symbols are thought to be "wildcard elements" which MUST have their atomic numbers
// equal to minus one (that is, NOT_DEFINED). if this logic is changed, some bad problems will
// emerge in the ring search and in typerules...

element::element(char * p1)
{
	i32s n1 = 0;
	while (n1 < ELEMENT_SYMBOLS && strcmp(symbol[n1], p1)) n1++;
	
	if (n1 < ELEMENT_SYMBOLS) atomic_number = n1 + 1;
	else atomic_number = NOT_DEFINED;
}

element::element(i32s p1)
{
	atomic_number = p1;
}

element::~element(void)
{
}

const char * element::GetString(void) const
{
	if (atomic_number < 1) return "???";
	else return string[atomic_number - 1];
}

i32s element::GetAtomicNumber(void) const
{
	return atomic_number;
}

// if atomic number is not reasonable return "??".
// what is the number for dummy atom??? and sumbol ??? how about "**" ???

const char * element::GetSymbol(void) const
{
	if (atomic_number < 1) return "??";
	else return symbol[atomic_number - 1];
}

const fGL * element::GetColor(void) const
{
	if (atomic_number < 1) return NULL;
	else return color[atomic_number - 1];
}

fGL element::GetAtomicMass(void) const
{
	if (atomic_number < 1) return -1.0;
	else return mass[atomic_number - 1];
}

fGL element::GetVDWRadius(void) const
{
	if (atomic_number < 1) return -1.0;
	else return vdwr[atomic_number - 1];
}

void element::operator++(void)
{
	if (++atomic_number > ELEMENT_SYMBOLS) atomic_number = 1;
}

void element::operator--(void)
{
	if (--atomic_number < 1) atomic_number = ELEMENT_SYMBOLS;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

bondtype bondtype::current_bondtype = bondtype('S');

const char * bondtype::string[4] =
{
	"cnjgtd", "single", "double", "triple"
};

const char bondtype::symbol1[4] = { 'C', 'S', 'D', 'T' };
const char bondtype::symbol2[4] = { '~', '-', '=', '#' };

bondtype::bondtype(void)
{
	type = NOT_DEFINED;
}

bondtype::bondtype(char p1)
{
	type = NOT_DEFINED;
	while (type++ < 3)
	{
		if (symbol1[type] == p1) return;
		if (symbol2[type] == p1) return;
	}
	
	type = NOT_DEFINED;
}

bondtype::bondtype(i32s p1)
{
	type = p1;
}

bondtype::~bondtype(void)
{
}

const char * bondtype::GetString(void) const
{
	return string[type];
}

i32s bondtype::GetValue(void) const
{
	return type;
}

char bondtype::GetSymbol1(void) const
{
	if (type < 0)
	{
		cout << "trying to use invalid bondtype..." << endl;
		return 'S';
	}
	
	return symbol1[type];
}

char bondtype::GetSymbol2(void) const
{
	if (type < 0)
	{
		cout << "trying to use invalid bondtype..." << endl;
		return '-';
	}
	
	return symbol2[type];
}

void bondtype::operator++(void)
{
	if (++type > 3) type = 0;
}

void bondtype::operator--(void)
{
	if (--type < 0) type = 3;
}

ostream & operator<<(ostream & p1, bondtype & p2)
{
	switch (p2.type)
	{
		case BT_CNJGTD:		p1 << "cnjgtd"; break;
		case BT_SINGLE:		p1 << "single"; break;
		case BT_DOUBLE:		p1 << "double"; break;
		case BT_TRIPLE:		p1 << "triple"; break;
		default:		p1 << "???";
	}
	
	return p1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
