// UTILITY.H : some utility functions and classes...

// Copyright (C) 1998 Tommi Hassinen, Jarno Huuskonen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef UTILITY_H
#define UTILITY_H

class element;
class bondtype;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "v3d.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

void TransformVector(v3d<fGL> &, const fGL *);

////////////////////////////////////////////////////////////////////////////////////////////////////

#define ELEMENT_SYMBOLS 105

class element
{
	private:
	
	i32s atomic_number;
	
	static const char * string[ELEMENT_SYMBOLS];
	static const char symbol[ELEMENT_SYMBOLS][3];
	
	static const fGL color[ELEMENT_SYMBOLS][3];
	
	static const fGL mass[ELEMENT_SYMBOLS];
	static const fGL vdwr[ELEMENT_SYMBOLS];
	
	public:
	
	static element current_element;		// is this for GUI??? why it's here???
	
	public:
	
	element(void);
	element(char *);	// symbols
	element(i32s);		// atomic numbers
	~element(void);
	
	const char * GetString(void) const;
	
	i32s GetAtomicNumber(void) const;
	const char * GetSymbol(void) const;
	
	const fGL * GetColor(void) const;
	
	fGL GetAtomicMass(void) const;
	fGL GetVDWRadius(void) const;
	
	void operator++(void);
	void operator--(void);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#define BT_CNJGTD 0
#define BT_SINGLE 1
#define BT_DOUBLE 2
#define BT_TRIPLE 3

class bondtype
{
	private:
	
	i32s type;
	
	static const char * string[4];
	
	static const char symbol1[4];
	static const char symbol2[4];
	
	public:
	
	static bondtype current_bondtype;	// is this for GUI??? why it's here???
	
	public:
	
	bondtype(void);
	bondtype(char);		// symbols
	bondtype(i32s);		// number codes
	~bondtype(void);
	
	const char * GetString(void) const;
	
	i32s GetValue(void) const;
	char GetSymbol1(void) const;
	char GetSymbol2(void) const;
	
	void operator++(void);
	void operator--(void);
	
	friend ostream & operator<<(ostream &, bondtype &);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

template<class TYPE1> class singleton_cleaner
{
	private:
	
	TYPE1 * instance;
	
	public:
	
	singleton_cleaner(TYPE1 * p1 = NULL) { instance = p1; }
	~singleton_cleaner(void) { if (instance) delete instance; }
	
	bool SetInstance(TYPE1 * p1) { if (instance) return false; instance = p1; return true; }
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// UTILITY_H

// eof
