// MM1GUI.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "mm1gui.h"

#ifdef ENABLE_GUI
#include "mm1alg.h"
#include "mm1eng1.h"
#include "mm1eng9.h"

#include "mm1rbn.h"

#include "plane.h"
#include "surface.h"

#include "color.h"
#include "window.h"

#include <fstream>
#include <strstream>

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET3

GnomeUIInfo mm1_gui::file_submenu[] =
{
	GNOMEUIINFO_MENU_SAVE_AS_ITEM(GtkGLPopupHandler, SIGNAL_MM1_FILE_SAVEAS),
	GNOMEUIINFO_MENU_CLOSE_ITEM(GtkGLPopupHandler, SIGNAL_MM1_FILE_CLOSE),
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::select_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Select All", "select all atoms...",
		GtkGLPopupHandler, (void *) SIGNAL_GEN_SELECT_SELECTALL, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Invert Selection", "invert the selection...",
		GtkGLPopupHandler, (void *) SIGNAL_GEN_SELECT_INVERTSELECTION, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::settings_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Set Current Element", "el...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_SETTINGS_ELEMENT, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Set Current Bondtype", "bt...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_SETTINGS_BONDTYPE, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::viewscreate_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "New Camera", "create a new view with a new camera...",
		GtkGLPopupHandler, (void *) SIGNAL_GEN_VIEWS_NEWCAMERA, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "New View", "create a new view using the current camera...",
		GtkGLPopupHandler, (void *) SIGNAL_GEN_VIEWS_NEWVIEW, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::views_submenu[] =
{
	GNOMEUIINFO_SUBTREE("_Create", viewscreate_submenu),
	
	{
		GNOME_APP_UI_ITEM, "Delete This View", "remove this view...",
		GtkGLPopupHandler, (void *) SIGNAL_GEN_VIEWS_DELETEVIEW, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::lights_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Create New Light", "create a new light object...",
		GtkGLPopupHandler, (void *) SIGNAL_GEN_LIGHTS_NEWLIGHT, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::rendermode_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Wireframe", "wf",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_RMODE_WIREFRAME, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Ball & Stick", "bs",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_RMODE_BALLANDSTICK, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "vanderWaals", "vdw",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_RMODE_VANDERWAALS, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Cylinders", "cyl",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_RMODE_CYLINDERS, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Nothing", "null",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_RMODE_NOTHING, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::projection_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Orthographic", "ortho",
		GtkGLPopupHandler, (void *) SIGNAL_GEN_PROJECTION_ORTHOGRAPHIC, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Perspective", "persp",
		GtkGLPopupHandler, (void *) SIGNAL_GEN_PROJECTION_PERSPECTIVE, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::render_submenu[] =
{
	GNOMEUIINFO_SUBTREE("_Views", views_submenu),
	GNOMEUIINFO_SUBTREE("_Lights", lights_submenu),
	GNOMEUIINFO_SUBTREE("_Rendering Mode", rendermode_submenu),
	GNOMEUIINFO_SUBTREE("_Projection", projection_submenu),
	
	{
		GNOME_APP_UI_ITEM, "Quick Update Mode", "quick update...",
		GtkGLPopupHandler, (void *) SIGNAL_GEN_RENDER_QUICKUPDATE, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::objects_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Ribbon", "...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_OBJECTS_RIBBON, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "ESP-plane", "...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_OBJECTS_EPLANE, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "ESP-surface", "...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_OBJECTS_ESURFACE, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "ESP-colored vdW-surface", "...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_OBJECTS_VDWSURFACE, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::compute_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Geometry Optimization...", "...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_COMPUTE_GEOMOPT, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Molecular Dynamics...", "...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_COMPUTE_MOLDYN, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo mm1_gui::popupmenu_info[] =
{
	// a "tearoff" item would be nice here at the beginning !?!?!?!??!?!?!?!?!?!?!?!!!!???!??!!!
	// a "tearoff" item would be nice here at the beginning !?!?!?!??!?!?!?!?!?!?!?!!!!???!??!!!
	// a "tearoff" item would be nice here at the beginning !?!?!?!??!?!?!?!?!?!?!?!!!!???!??!!!
	
	GNOMEUIINFO_SUBTREE("_File", file_submenu),
	GNOMEUIINFO_SUBTREE("_Select", select_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE("_Settings...", settings_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE("_Render", render_submenu),
	GNOMEUIINFO_SUBTREE("_Objects", objects_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE("_Compute", compute_submenu),
	GNOMEUIINFO_SEPARATOR,
	
	{
		GNOME_APP_UI_ITEM, "Hydrogens...", "...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_HYDROGENS, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Sequence Builder...", "...",
		GtkGLPopupHandler, (void *) SIGNAL_MM1_BUILDER, NULL
	},
	
	GNOMEUIINFO_END
};

#endif	// TARGET3

mm1_cm_element mm1_gui::cm_element = mm1_cm_element();
mm1_cm_state mm1_gui::cm_state = mm1_cm_state();

mm1_gui::mm1_gui(window_factory & p1, ostream * p2) : bs_gui(p1, p2), mm1_mdl(p2), bs_mdl_simple(p2)
{
	draw_box = !true;
	draw_images = !true;
	
#ifdef TARGET3

	popupmenu = gnome_popup_menu_new(popupmenu_info);
	
#endif	// TARGET3

}

mm1_gui::~mm1_gui(void)
{
}

const char * mm1_gui::GetType(void)
{
	return "MM, all-atoms model";
}

color_mode * mm1_gui::GetDefaultColorMode(void)
{
	return & mm1_gui::cm_element;
}

void mm1_gui::SelectAll(void)
{
	iter_mm1al it1 = atom_list.begin();
	while (it1 != atom_list.end()) (* it1++).selected = true;
}

void mm1_gui::InvertSelection(void)
{
	iter_mm1al it1 = atom_list.begin();
	while (it1 != atom_list.end())
	{
		bool flag = (* it1).selected;
		(* it1++).selected = !flag;
	}
}

bool mm1_gui::TestAtom(mm1_atom * ref, rmode rm)
{
	// what about the "visible"-flag???
	
	if (rm == Transform1 && ref->selected) return false;
	if (rm == Transform2 && !ref->selected) return false;
	
	return true;
}

bool mm1_gui::TestBond(mm1_bond * ref, rmode rm)
{
	// what about the "visible"-flag???
	
	if (rm == Transform1 && ref->atmr[0]->selected) return false;
	if (rm == Transform2 && !ref->atmr[0]->selected) return false;
	
	bool test = (ref->atmr[0]->selected != ref->atmr[1]->selected);
	if (rm != Normal && test) return false;
	
	return true;
}

void mm1_gui::SetColor(color_mode * cm, mm1_atom * ref)
{
	if (ref->selected) glColor3f(1.0, 0.0, 1.0);
	else
	{
		fGL_a3 color;
		cm->GetColor(ref, color);
		glColor3fv(color);
	}
}

// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!

void mm1_gui::InitGL(void)
{
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glDepthFunc(GL_LESS); glEnable(GL_DEPTH_TEST);
	
	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 64);
	const fGL specular_reflectance[4] = { 0.9, 0.9, 0.9, 1.0 };
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular_reflectance);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	
	const fGL ambient_intensity[4] = { 0.2, 0.2, 0.2, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_intensity);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, true);
	
	glFogi(GL_FOG_MODE, GL_EXP); glFogf(GL_FOG_DENSITY, 0.15);
	const fGL fog_color[4] = { 0.0, 0.0, 0.0, 0.0 };
	glFogfv(GL_FOG_COLOR, fog_color);
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glSelectBuffer(SB_SIZE, select_buffer);
}

#define BALL 0.25	// the ball radius
#define STICK 0.01	// the stick radius

#define CYLRAD 0.035	// the cylinder radius

// selection is done in a rather crude way by dumping the atom pointers directly to
// the OGL selection buffer and reading them later there... works fine as long as
// both datatypes have the same size (currently it's 32 bits). if we some day need
// 64-bit pointers and have only 32-bit sel-buffer we have to use two names...

void mm1_gui::Render(view_window * win, rmode rm)
{
	bool accum = win->accumulate; if (rm != Normal) accum = false;
	if (accum) { glClear(GL_ACCUM_BUFFER_BIT); UpdateAccumValues(); }
	else if (rm != Transform2) glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	if (draw_box && rm == Normal)
	{
		glLineWidth(1.0);
		glColor3f(1.0, 0.0, 1.0);
		glBegin(GL_LINES);
		
		glVertex3f(-box_hdim[0], -box_hdim[1], +box_hdim[2]);
		glVertex3f(+box_hdim[0], -box_hdim[1], +box_hdim[2]);
		
		glVertex3f(-box_hdim[0], +box_hdim[1], +box_hdim[2]);
		glVertex3f(+box_hdim[0], +box_hdim[1], +box_hdim[2]);
		
		glVertex3f(-box_hdim[0], -box_hdim[1], -box_hdim[2]);
		glVertex3f(+box_hdim[0], -box_hdim[1], -box_hdim[2]);
		
		glVertex3f(-box_hdim[0], +box_hdim[1], -box_hdim[2]);
		glVertex3f(+box_hdim[0], +box_hdim[1], -box_hdim[2]);
		
		glVertex3f(-box_hdim[0], -box_hdim[1], -box_hdim[2]);
		glVertex3f(-box_hdim[0], -box_hdim[1], +box_hdim[2]);
		
		glVertex3f(-box_hdim[0], +box_hdim[1], -box_hdim[2]);
		glVertex3f(-box_hdim[0], +box_hdim[1], +box_hdim[2]);
		
		glVertex3f(+box_hdim[0], -box_hdim[1], -box_hdim[2]);
		glVertex3f(+box_hdim[0], -box_hdim[1], +box_hdim[2]);
		
		glVertex3f(+box_hdim[0], +box_hdim[1], -box_hdim[2]);
		glVertex3f(+box_hdim[0], +box_hdim[1], +box_hdim[2]);
		
		glVertex3f(-box_hdim[0], -box_hdim[1], -box_hdim[2]);
		glVertex3f(-box_hdim[0], +box_hdim[1], -box_hdim[2]);
		
		glVertex3f(-box_hdim[0], -box_hdim[1], +box_hdim[2]);
		glVertex3f(-box_hdim[0], +box_hdim[1], +box_hdim[2]);
		
		glVertex3f(+box_hdim[0], -box_hdim[1], -box_hdim[2]);
		glVertex3f(+box_hdim[0], +box_hdim[1], -box_hdim[2]);
		
		glVertex3f(+box_hdim[0], -box_hdim[1], +box_hdim[2]);
		glVertex3f(+box_hdim[0], +box_hdim[1], +box_hdim[2]);
		
		glEnd();
	}
	
	if (win->enable_fog) glEnable(GL_FOG);
	
	i32s layers = 0;
	if (draw_images && rm == Normal) layers = 1;
	
	for (i32s r1 = -layers;r1 < (layers + 1);r1++)
	{
		for (i32s r2 = -layers;r2 < (layers + 1);r2++)
		{
			for (i32s r3 = -layers;r3 < (layers + 1);r3++)
			{
				glPushMatrix();
				
				fGL trans1 = r1 * box_fdim[0];
				fGL trans2 = r2 * box_fdim[1];
				fGL trans3 = r3 * box_fdim[2];
				
				glTranslated(trans1, trans2, trans3);
				
				RenderScene(win, rm, accum);
				
				glPopMatrix();
			}
		}
	}
	
	if (accum) glAccum(GL_RETURN, 1.0);
	else if (rm != Transform2) win->cam->RenderObjects(win);
	
	if (win->label == LABEL_ELEMENT)
	{
		glDisable(GL_DEPTH_TEST); glColor3f(1.0, 1.0, 1.0); char string[32];
		for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
		{
			glRasterPos3fv((* it1).crd_vector[0].data);
			
			ostrstream str(string, sizeof(string));
			str << (* it1).el.GetSymbol() << ends;
			win->WriteBitmapString(GLUT_BITMAP_9_BY_15, string);
		}
		glEnable(GL_DEPTH_TEST);  
	}
	
	if (win->label == LABEL_BONDTYPE)
	{
		glDisable(GL_DEPTH_TEST); glColor3f(1.0, 1.0, 1.0); char string[32];
		for (iter_mm1bl it1 = bond_list.begin();it1 != bond_list.end();it1++)
		{
			fGL rpos[3];
			for (i32s n1 = 0;n1 < 3;n1++)
			{
				fGL tmp1 = (* it1).atmr[0]->crd_vector[0][n1];
				fGL tmp2 = (* it1).atmr[1]->crd_vector[0][n1];
				rpos[n1] = (tmp1 + tmp2) / 2.0;
			} glRasterPos3fv(rpos);
			
			ostrstream str(string, sizeof(string));
			str << (* it1).bt.GetSymbol1() << ends;
			win->WriteBitmapString(GLUT_BITMAP_9_BY_15, string);
		}
		glEnable(GL_DEPTH_TEST);  
	}
	
	if (win->enable_fog) glDisable(GL_FOG);
}

// this is just quickly cut into a different function for clarity...
// this is just quickly cut into a different function for clarity...
// this is just quickly cut into a different function for clarity...

void mm1_gui::RenderScene(view_window * win, rmode rm, bool accum)
{
	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
	{
		if (!cs_vector[n1].visible) continue;
		if (accum) glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
		if (win->render == RENDER_WIREFRAME)
		{
			glPointSize(3.0); glLineWidth(1.0);
			for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
			{
				if (!TestAtom(& (* it1), rm)) continue;
				glPushName(GLNAME_MD_TYPE1); glPushName((i32u) & (* it1));
				
				glBegin(GL_POINTS);
				SetColor(win->colormode, & (* it1));
				glVertex3fv((* it1).crd_vector[n1].data);
				glEnd();
				
				glPopName(); glPopName();
			}
			
			glEnable(GL_LINE_STIPPLE);
			for (iter_mm1bl it2 = bond_list.begin();it2 != bond_list.end();it2++)
			{
				if (!TestBond(& (* it2), rm)) continue;
				
				switch ((* it2).bt.GetSymbol1())
				{
					case 'S': glLineStipple(1, 0xFFFF); break;
					case 'C': glLineStipple(1, 0x3FFF); break;
					case 'D': glLineStipple(1, 0x3F3F); break;
					case 'T': glLineStipple(1, 0x3333); break;
				}
				
				glBegin(GL_LINES);
				SetColor(win->colormode, (* it2).atmr[0]);
				glVertex3fv((* it2).atmr[0]->crd_vector[n1].data);
				SetColor(win->colormode, (* it2).atmr[1]);
				glVertex3fv((* it2).atmr[1]->crd_vector[n1].data);
				glEnd();
			}
			
			glDisable(GL_LINE_STIPPLE);
		}
		
		if (win->render != RENDER_WIREFRAME && win->render != RENDER_NOTHING)
		{
			glEnable(GL_LIGHTING);
			
			for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
			{
				if (!TestAtom(& (* it1), rm)) continue;
				
				SetColor(win->colormode, & (* it1));
				
				float rad = 0.0; int res = 0;
				switch (win->render)
				{
					case RENDER_BALL_AND_STICK:
					rad = BALL * (* it1).el.GetVDWRadius(); res = 12;
					break;
					
					case RENDER_VAN_DER_WAALS:
					rad = (* it1).el.GetVDWRadius(); res = 12;
					break;
					
					case RENDER_CYLINDERS:
					rad = CYLRAD; res = 12;
					break;
				}
				
				glPushName(GLNAME_MD_TYPE1); glPushName((i32u) & (* it1));
				
				GLUquadricObj * qo = gluNewQuadric();
				gluQuadricDrawStyle(qo, (GLenum) GLU_FILL);
				glPushMatrix();
				glTranslated((* it1).crd_vector[n1][0], (* it1).crd_vector[n1][1], (* it1).crd_vector[n1][2]);
				gluSphere(qo, rad, res, res / 2);
				glPopMatrix();
				gluDeleteQuadric(qo);
				
				glPopName(); glPopName();
			}
			
			glDisable(GL_LIGHTING);
		}
		
		if (win->render == RENDER_BALL_AND_STICK || win->render == RENDER_CYLINDERS)
		{
			glEnable(GL_LIGHTING);
			
			for (iter_mm1bl it1 = bond_list.begin();it1 != bond_list.end();it1++)
			{
				if (!TestBond(& (* it1), rm)) continue;
				
				fGL vdwr[2] =
				{
					(* it1).atmr[0]->el.GetVDWRadius(),
					(* it1).atmr[1]->el.GetVDWRadius()
				};
				
				fGL vdwrsum = vdwr[0] + vdwr[1];
				
				for (i32s n2 = 0;n2 < 2;n2++)
				{
					fGL * crd1 = (* it1).atmr[n2]->crd_vector[n1].data;
					fGL * crd2 = (* it1).atmr[!n2]->crd_vector[n1].data;
					v3d<fGL> crt = v3d<fGL>(crd1, crd2);
					
					fGL pol[3]; crt2pol(crt.data, pol);
					
					SetColor(win->colormode, (* it1).atmr[n2]);
					
					float rad = 0.0; int res = 0;
					switch (win->render)
					{
						case RENDER_BALL_AND_STICK:
						rad = 0.01; res = 12;
						break;
						
						case RENDER_CYLINDERS:
						rad = CYLRAD; res = 12;
						break;
					}
					
					glPushName(GLNAME_MD_TYPE1); glPushName((i32u) (* it1).atmr[n2]);
					
					GLUquadricObj * qo = gluNewQuadric();
					gluQuadricDrawStyle(qo, (GLenum) GLU_FILL);
					glPushMatrix();
					
					glTranslated(crd1[0], crd1[1], crd1[2]);
					
					glRotated(180.0 * pol[1] / M_PI, 0.0, 1.0, 0.0);
					glRotated(180.0 * pol[2] / M_PI, sin(-pol[1]), 0.0, cos(-pol[1]));
					
					fGL length = crt.len() * vdwr[n2] / vdwrsum;
					gluCylinder(qo, rad, rad, length, res, res / 2);
					
					glPopMatrix();
					gluDeleteQuadric(qo);
					
					glPopName(); glPopName();
				}
			}
			
			glDisable(GL_LIGHTING);
		}
		
		if (accum)
		{
			win->cam->RenderObjects(win);
			glAccum(GL_ACCUM, cs_vector[n1].accum_value);
		}
	}
}

void mm1_gui::Center(transformer * p1)
{
	i32s sum = 0;
	p1->GetLocDataRW()->crd[0] = 0.0;
	p1->GetLocDataRW()->crd[1] = 0.0;
	p1->GetLocDataRW()->crd[2] = 0.0;
	
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		if (!(* it1).selected) continue;
		for (i32u n1 = 0;n1 < cs_vector.size();n1++)
		{
			sum++;
			p1->GetLocDataRW()->crd[0] += (* it1).crd_vector[n1][0];
			p1->GetLocDataRW()->crd[1] += (* it1).crd_vector[n1][1];
			p1->GetLocDataRW()->crd[2] += (* it1).crd_vector[n1][2];
		}
	}
	
	if (!sum) return;
	
	p1->GetLocDataRW()->crd[0] /= (fGL) sum;
	p1->GetLocDataRW()->crd[1] /= (fGL) sum;
	p1->GetLocDataRW()->crd[2] /= (fGL) sum;
	
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		if (!(* it1).selected) continue;
		for (i32u n1 = 0;n1 < cs_vector.size();n1++)
		{
			(* it1).crd_vector[n1][0] -= p1->GetLocData()->crd[0];
			(* it1).crd_vector[n1][1] -= p1->GetLocData()->crd[1];
			(* it1).crd_vector[n1][2] -= p1->GetLocData()->crd[2];
		}
	}
}

void mm1_gui::Transform(transformer * p1)
{
	fGL matrix[16]; p1->GetMatrix(matrix);
	
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		if (!(* it1).selected) continue;
		
		for (i32u n1 = 0;n1 < cs_vector.size();n1++)
		{
			v3d<fGL> posv = v3d<fGL>((* it1).crd_vector[n1].data);
			TransformVector(posv, matrix);
			
			(* it1).crd_vector[n1][0] = posv[0];
			(* it1).crd_vector[n1][1] = posv[1];
			(* it1).crd_vector[n1][2] = posv[2];
		}
	}
}

void mm1_gui::DrawEvent(view_window * win, vector<iGLu> & names)
{
	if (window::button == mouse_tool::Right) return;	// the right button is for popup menus...
	
	i32s mouse[2] =
	{
		win->current_tool->latest_x,
		win->current_tool->latest_y
	};
	
	if (window::state == mouse_tool::Down)
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[0] = (mm1_atom *) names[1];
		}
		else
		{
			fGL tmp1[3]; win->GetCRD(mouse, tmp1);
			mm1_atom newatom(element::current_element, tmp1, cs_vector.size());
			AddAtom(newatom); draw_data[0] = & atom_list.back();
		}
	}
	else
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[1] = (mm1_atom *) names[1];
		}
		else
		{
			fGL tmp1[3]; win->GetCRD(mouse, tmp1);
			mm1_atom newatom(element::current_element, tmp1, cs_vector.size());
			AddAtom(newatom); draw_data[1] = & atom_list.back();
		}
		
		// if different: update bondtype or add a new bond.
		// if not different: change atom to different element.
		
		if (draw_data[0] != draw_data[1])
		{
			mm1_bond newbond(draw_data[0], draw_data[1], bondtype::current_bondtype);
			iter_mm1bl it1 = find(bond_list.begin(), bond_list.end(), newbond);
			if (it1 != bond_list.end()) (* it1).bt = bondtype::current_bondtype;
			else AddBond(newbond);
		}
		else
		{
			draw_data[0]->el = element::current_element;
		}
		
		UpdateAllViewWindows();
	}
}

void mm1_gui::EraseEvent(view_window * win, vector<iGLu> & names)
{
	if (window::button == mouse_tool::Right) return;	// the right button is for popup menus...
	
	if (window::state == mouse_tool::Down)
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[0] = (mm1_atom *) names[1];
		}
		else
		{
			draw_data[0] = NULL;
		}
	}
	else
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[1] = (mm1_atom *) names[1];
		}
		else
		{
			draw_data[1] = NULL;
		}
		
		if (!draw_data[0] || !draw_data[1]) return;
		
		// if different: try to find and remove a bond.
		// if not different: remove atom.
		
		if (draw_data[0] != draw_data[1])
		{
			mm1_bond tmpbond(draw_data[0], draw_data[1], bondtype::current_bondtype);
			iter_mm1bl it1 = find(bond_list.begin(), bond_list.end(), tmpbond);
			if (it1 != bond_list.end()) RemoveBond(it1); else return;
		}
		else
		{
			iter_mm1al it1 = find(atom_list.begin(), atom_list.end(), (* draw_data[0]));
			if (it1 != atom_list.end()) RemoveAtom(it1); else exit(EXIT_FAILURE);
		}
		
		UpdateAllViewWindows();
	}
}

void mm1_gui::SelectEvent(view_window *, vector<iGLu> & names)
{
	if (names[0] == GLNAME_MD_TYPE1)
	{
		mm1_atom * ref = (mm1_atom *) names[1];
		ref->selected = !ref->selected;
		UpdateAllViewWindows();
		
		cout << "atomtype is " << hex << ref->atmtp << dec << endl;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET2

#define CFRQ 1
#define CSIZE 100

#define START 10000

static ifstream * traj = NULL;
static i32s nframes = NOT_DEFINED;
static i32s fcount = NOT_DEFINED;

void mm1_gui::KeyEvent(view_window * win, char key, int, int)
{
	i32s n1; i32s n2;
	
	i32s ccnt = 0;
	f64 control[CSIZE];
	
	mm1_eng * eng;
	mm1_eng_exp1b3 * yyy = NULL;		// temporary...
	mm1_eng_pbc * zzz = NULL;		// temporary...
	
	mm1_geomopt * opt;
	mm1_moldyn * dyn;
	
	ifstream ifile;
	ofstream ofile;
	
	char input;
	
	char seq[1024];
	f64 aab_ah[] = { 302.0 * M_PI / 180.0, 313.0 * M_PI / 180.0, M_PI };
	f64 aab_bs[] = { 180.0 * M_PI / 180.0, 180.0 * M_PI / 180.0, M_PI };
	
	f64 nab[] =
	{
		44.6 * M_PI / 180.0, 261.0 * M_PI / 180.0, 320.8 * M_PI / 180.0, 208.6 * M_PI / 180.0,
		273.8 * M_PI / 180.0, 105.6 * M_PI / 180.0, 356.0 * M_PI / 180.0, 24.7 * M_PI / 180.0,
		88.7 * M_PI / 180.0, 264.6 * M_PI / 180.0
	};
	
	fGL tmp1; fGL tmp2;
	i32s sz[3] = { 50, 50, 50 };
	fGL dm[3] = { 0.55, 0.50, 0.45 };
	
	cp_param cpp;
	cpp.np = 30; cpp.gui = this; cpp.ref = this;
	
	cs_param csp;
	csp.np = sz; csp.dim = dm; csp.ostr = & cout;
	csp.gui = this; csp.ref = this;
	
	switch (key)
	{
		case 'h':
		cout << "+ -       -> change the current element" << endl;
		cout << "< >       -> change the current bondtype" << endl;
		cout << "b B       -> build sequences (amino/nucleic acids)" << endl;
		cout << "C         -> change color modes" << endl;
		cout << "r         -> change rendering modes (without updating view)" << endl;
		cout << "Q         -> change labels" << endl;
		cout << "i         -> import HIN/ENT files" << endl;
		cout << "O         -> open a file" << endl;
		cout << "S         -> save to a file" << endl;
		cout << "H         -> hydrogens, add or remove" << endl;
		cout << "e n m     -> make some nice planes/surfaces..." << endl;
		cout << "t         -> build a protein ribbon model" << endl;
		cout << "z         -> compare numerical and analytical gradient" << endl;
		cout << "E         -> compute energy [kJ/mol]" << endl;
		cout << "o         -> geometry optimization..." << endl;
		cout << "d         -> molecular dynamics..." << endl;
		cout << "X         -> trajectory mode on/off" << endl;
		cout << "1 2 7     -> (in traj-mode) get prev/next/desired frame" << endl;
		cout << "5         -> (in traj-mode) play the trajectory file" << endl;
		break;
		
		case '+': ++element::current_element;
		cout << "current element = " << element::current_element.GetAtomicNumber();
		cout << " " << element::current_element.GetSymbol() << endl;
		break;
		
		case '-': --element::current_element;
		cout << "current element = " << element::current_element.GetAtomicNumber();
		cout << " " << element::current_element.GetSymbol() << endl;
		break;
		
		case '<': ++bondtype::current_bondtype;
		cout << "current bondtype = " << bondtype::current_bondtype << endl;
		break;
		
		case '>': --bondtype::current_bondtype;
		cout << "current bondtype = " << bondtype::current_bondtype << endl;
		break;
		
		case 'b':
		cout << "give the sequence using symbols ARNDCQEGHILKMFPSTWYV ???"; cin >> seq;
		cout << "please select the conformation : H = helix, S = sheet ???"; cin >> input;
		mm1_mdl::amino_builder.Build(this, seq, input == 'h' || input == 'H' ? aab_ah : aab_bs);
		UpdateAllViewWindows();
		break;
		
		case 'B':
		cout << "give the sequence using symbols AGTC and aguc ???"; cin >> seq;
		mm1_mdl::nucleic_builder.Build(this, seq, nab);
		UpdateAllViewWindows();
		break;
		
		case 'C':
		if (win->colormode == & mm1_gui::cm_element) win->colormode = & mm1_gui::cm_state;
		else if (win->colormode == & mm1_gui::cm_state) win->colormode = & mm1_gui::cm_element;
		UpdateAllViewWindows();
		break;
		
		case 'r':
		cout << "rendering = ";
		switch (win->render)
		{
			case RENDER_NOTHING:
			win->render = RENDER_WIREFRAME;
			cout << "RENDER_WIREFRAME";
			break;
			
			case RENDER_WIREFRAME:
			win->render = RENDER_BALL_AND_STICK;
			cout << "RENDER_BALL_AND_STICK";
			break;
			
			case RENDER_BALL_AND_STICK:
			win->render = RENDER_VAN_DER_WAALS;
			cout << "RENDER_VAN_DER_WAALS";
			break;
			
			case RENDER_VAN_DER_WAALS:
			win->render = RENDER_CYLINDERS;
			cout << "RENDER_CYLINDERS";
			break;
			
			case RENDER_CYLINDERS:
			win->render = RENDER_NOTHING;
			cout << "RENDER_NOTHING";
			break;
		}
		cout << endl;
		break;
		
		case 'Q':
		cout << "label = ";
		switch (win->label)
		{
			case LABEL_NOTHING:
			win->label = LABEL_ELEMENT;
			cout << "LABEL_ELEMENT";
			break;
			
			case LABEL_ELEMENT:
			win->label = LABEL_BONDTYPE;
			cout << "LABEL_BONDTYPE";
			break;
			
			case LABEL_BONDTYPE:
			win->label = LABEL_NOTHING;
			cout << "LABEL_NOTHING";
			break;
		}
		cout << endl;
		break;
		
		case 'i':
		cout << "type of file to import (HIN or ENT) ? (h/e) ";
		cin >> input; if (input != 'h' && input != 'e') break;
		
		cout << "name of file to import ??? ";
		cin >> buffer;
		
		if (input == 'h') ImportHIN(buffer);
		if (input == 'e') ImportENT(buffer, NOT_DEFINED);
		
		UpdateAllViewWindows();
		break;
		
		case 'O':
		cout << "name of file to open ??? "; cin >> buffer;
		ifile.open(buffer, ios::in); ReadStream(ifile); ifile.close();
		UpdateAllViewWindows();
		break;
		
		case 'S':
		cout << "name of file to save ??? "; cin >> buffer;
		ofile.open(buffer, ios::out); WriteStream(ofile); ofile.close();
		break;
		
		case 'H':
		cout << "hydrogens, add or remove ? (a/r) ";
		cin >> input;
		
		if (input == 'a')
		{
			cout << "adding hydrogens..." << endl;
			AddHydrogens(); UpdateAllViewWindows();
		}
		
		if (input == 'r')
		{
			cout << "removing hydrogens..." << endl;
			RemoveHydrogens(); UpdateAllViewWindows();
		}
		break;
		
		case 'e':
		cout << "give dimension of the plane [nm] (1.0 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) mm1_GetESPValue; cpp.cf = GetRGBRange2;
		cpp.value = 4.1868 * 33.20716; cpp.descr = "ESP-";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case 'n': tmp1 = 4.1868 * 33.20716;
		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) mm1_GetESPValue; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetRedColor; csp.svalue = +tmp1; csp.descr = "pos ESP-";
		AddObject(new color_surface(ol_static(), csp));
		
		csp.cf = GetBlueColor; csp.svalue = -tmp1; csp.descr = "neg ESP-";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
		case 'm': tmp1 = 4.1868 * 33.20716 / 2.0;
		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false; csp.cf = GetRGBRange2;
		csp.vf1 = (ValueFunction *) mm1_GetSASValue; csp.vf2 = (ValueFunction *) mm1_GetESPValue;
		csp.svalue = 1.0; csp.cvalue = tmp1; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		csp.descr = "ESP-colored VDW-";
		
		AddObject(new color_surface(ol_static(), csp));
		UpdateAllViewWindows();
		break;
				
		case 't':
		mm1_mdl::amino_builder.Identify(this);
		DefineSecondaryStructure(this);
		
		AddObject(new mm1_ribbon(this, win->colormode, 0, 4));		// min. order is 2!!!
		UpdateAllViewWindows();
		break;
		
		case 'z':
		eng = new mm1_eng_exp9(* this, 1, 0);		// DEFAULT ENGINE!!!
//		eng = new mm1_eng_exp1(* this, 1, 0);
		CopyCRD(this, eng, 0); eng->Check(1);
		delete eng;
		break;
		
		case 'E':
		eng = new mm1_eng_exp9(* this, 1, 0);		// DEFAULT ENGINE!!!
//		eng = new mm1_eng_exp1(* this, 1, 0);
//		eng = new mm1_eng_exp1_tst(* this, 1, 0);
//		GatherGroups(); eng = new mm1_eng_exp1_mim(* this, 1, 0);

		CopyCRD(this, eng, 0); eng->Compute(0);
		cout << "energy = " << eng->energy << " kJ/mol" << endl;
		delete eng;
		break;
		
		case 'o':
		cout << "how many cg-steps (500 might be Ok) ??? "; cin >> n1;
		
		eng = new mm1_eng_exp9(* this, 1, 0);		// DEFAULT ENGINE!!!
//		eng = new mm1_eng_exp1(* this, 1, 0);
//		GatherGroups(); eng = new mm1_eng_exp1_mim(* this, 1, 0);
		
		CopyCRD(this, eng, 0);
		opt = new mm1_geomopt(eng, 100, 0.025);		// optimal settings?!?!?
		
		for (n2 = 0;n2 < n1;n2++)
		{
			opt->TakeCGStep(conjugate_gradient::Newton2An);
			cout << n2 << " " << opt->optval << " " << opt->optstp << " " << endl;
			
			if (!(n2 % 10))
			{
				CopyCRD(eng, this, 0); CenterCRDSet(0);
				UpdateAllViewWindows(true);
			}
		}
		delete opt; delete eng;
		break;
		
		case 'd':
		cout << "how many md-steps for simulation (10000 might be Ok) ??? "; cin >> n1;
		
		cout << "there is " << START << " heating steps, total steps = " << (n1 + START) << endl;
		cout << "the trajectory file will contain " << (n1 / 100) << " frames!!!" << endl;
		
		cout << "time-step length [fs] (0.5 or less might be Ok) ??? "; cin >> tmp1;
		cout << "traj-file name ??? "; cin >> buffer;
		
		eng = new mm1_eng_exp9(* this, 1, 0);		// DEFAULT ENGINE!!!
//		eng = new mm1_eng_exp1(* this, 1, 0);
//		GatherGroups(); eng = zzz = yyy = new mm1_eng_exp1_mim(* this, 1, 0);
		
		CopyCRD(this, eng, 0); dyn = new mm1_moldyn(eng, 300.0, tmp1);
		
		ofile.open(buffer, ios::out | ios::binary);
		for (n2 = 0;n2 < n1 + START;n2++)
		{
			if (!(n2 % 1000) && zzz != NULL) zzz->CheckLocations();
			if (!(n2 % 100) && yyy != NULL) yyy->update = true;
			
			dyn->TakeMDStep();
			if (!(n2 % 50))
			{
				cout << n2 << " T = " << dyn->ConvEKinTemp(dyn->kin);
				cout << " pot = " << dyn->pot << " tot = " << (dyn->kin + dyn->pot) << endl;
			}
			
			if (n2 < START)
			{
				if (!(n2 % CFRQ))
				{
					control[ccnt++] = dyn->kin;
				}
				
				if (ccnt == CSIZE)
				{
					f64 sum = 0.0;
					for (i32s n9 = 0;n9 < ccnt;n9++)
					{
						sum += control[n9];
					}
					
					sum /= (f64) ccnt; ccnt = 0; f64 rtemp = (f64) (n2 + CFRQ) / (f64) START;
					cout << "average = " << sum << " kJ/mol = " << dyn->ConvEKinTemp(sum) << " K." << endl;
					dyn->SetEKin(dyn->kin + (dyn->ConvTempEKin(dyn->temp * rtemp) - sum));
				}
			}
			else if (!(n2 % 100))
			{
				CopyCRD(eng, this, 0);
				
				ofile.write((char *) & dyn->kin, sizeof(dyn->kin));
				ofile.write((char *) & dyn->pot, sizeof(dyn->pot));
				
				for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
				{
					for (i32s t4 = 0;t4 < 3;t4++)
					{
						f64 t1a = (* it1).crd_vector[0][t4];
						ofile.write((char *) & t1a, sizeof(t1a));
					}
				}
			}
			
			if (!(n2 % 100))
			{
				CopyCRD(eng, this, 0);
				UpdateAllViewWindows(true);
			}
		}
		
		delete dyn; delete eng;
		UpdateAccumValues();
		break;
		
		case 'X':
		if (traj != NULL)
		{
			cout << "traj mode off" << endl;
			traj->close(); delete traj; traj = NULL;
		}
		else
		{
			cout << "traj mode on" << endl;
			cout << "how many frames ??? "; cin >> nframes;
			cout << "traj-file name ??? "; cin >> buffer;
			traj = new ifstream(buffer, ios::in | ios::binary);
			fcount = 0;
		}
		break;
		
		case '1':
		case '2':
		case '7':
		if (traj != NULL)
		{
			switch (key)
			{
				case '1':
				fcount--; if (fcount < 0) fcount = 0;
				break;
				
				case '2':
				fcount++; if (fcount == nframes) fcount = nframes - 1;
				break;
				
				default:
				cout << "give the frame (in correct range) ??? "; cin >> fcount;
			}
			
			cout << "frame " << fcount << endl;
			
			i32s atoms = atom_list.size();
			i32s place = (2 + 3 * atoms) * sizeof(f64) * fcount;
			place += 2 * sizeof(f64);
			
			traj->seekg(place, ios::beg);
			
			for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
			{
				for (i32s t4 = 0;t4 < 3;t4++)
				{
					f64 t1a; traj->read((char *) & t1a, sizeof(t1a));
					(* it1).crd_vector[0][t4] = t1a;
				}
			}
			UpdateAllViewWindows();
		} else cout << "in traj-mode only!!!" << endl;
		break;
		
		case '5':
		if (traj != NULL)
		{
			i32s atoms = atom_list.size();
			for (i32s loop = 0;loop < nframes;loop++)
			{
				i32s place = (2 + 3 * atoms) * sizeof(f64) * loop;
				place += 2 * sizeof(f64);
				
				traj->seekg(place, ios::beg);
				
				for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
				{
					for (i32s t4 = 0;t4 < 3;t4++)
					{
						f64 t1a; traj->read((char *) & t1a, sizeof(t1a));
						(* it1).crd_vector[0][t4] = t1a;
					}
				}
				
				UpdateAllViewWindows(true);
			}
		} else cout << "in traj-mode only!!!" << endl;
		break;
		
		default:
		cout << "unknown command '" << key << "'" << endl;
	}
}

#endif	// TARGET2

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET3

#define CFRQ 1
#define CSIZE 100

#define START 10000

void mm1_gui::PopupEvent(view_window * win, int signal)
{
	char input;
	ofstream ofile;
	
	mm1_eng * eng;
	mm1_eng_exp1b3 * yyy = NULL;		// temporary...
	mm1_eng_pbc * zzz = NULL;		// temporary...
	
	mm1_geomopt * opt; mm1_moldyn * dyn;
	
	i32s n1; i32s n2;
	
	char seq[1024];
	f64 aab_ah[] = { 302.0 * M_PI / 180.0, 313.0 * M_PI / 180.0, M_PI };
	f64 aab_bs[] = { 180.0 * M_PI / 180.0, 180.0 * M_PI / 180.0, M_PI };
	
	f64 nab[] =
	{
		44.6 * M_PI / 180.0, 261.0 * M_PI / 180.0, 320.8 * M_PI / 180.0, 208.6 * M_PI / 180.0,
		273.8 * M_PI / 180.0, 105.6 * M_PI / 180.0, 356.0 * M_PI / 180.0, 24.7 * M_PI / 180.0,
		88.7 * M_PI / 180.0, 264.6 * M_PI / 180.0
	};
	
	i32s ccnt = 0;
	f64 control[CSIZE];
	
	fGL tmp1; fGL tmp2;
	i32s sz[3] = { 50, 50, 50 };
	fGL dm[3] = { 0.55, 0.50, 0.45 };
	
	cp_param cpp;
	cpp.np = 30; cpp.gui = this; cpp.ref = this;
	
	cs_param csp;
	csp.np = sz; csp.dim = dm; csp.ostr = & cout;
	csp.gui = this; csp.ref = this;
	
	switch (signal)
	{
		case SIGNAL_MM1_FILE_SAVEAS:
		cout << "name of file to save ??? "; cin >> buffer;
		ofile.open(buffer, ios::out); WriteStream(ofile); ofile.close();
		break;
		
		case SIGNAL_MM1_FILE_CLOSE:
		cout << "this should close all windows and delete the model." << endl;
		cout << "however, it's not yet implemented :) Sorry!!!" << endl;
		break;
		
		case SIGNAL_MM1_SETTINGS_ELEMENT:
		cout << "give the atomic number for current element (1-" << ELEMENT_SYMBOLS << ") ??? ";
		cin >> n1; element::current_element = element(n1);
		
		cout << "current element = " << element::current_element.GetAtomicNumber();
		cout << " " << element::current_element.GetSymbol() << endl;
		break;
		
		case SIGNAL_MM1_SETTINGS_BONDTYPE:
		cout << "give the symbol for current bondtype (S, D, T, C) ??? ";
		cin >> input; bondtype::current_bondtype = bondtype(input);
		
		cout << "current bondtype = " << bondtype::current_bondtype << endl;
		break;
		
		case SIGNAL_MM1_HYDROGENS:
		cout << "hydrogens, add or remove ? (a/r) "; cin >> input;
		
		if (input == 'a')
		{
			cout << "adding hydrogens..." << endl;
			AddHydrogens(); UpdateAllViewWindows();
		}
		
		if (input == 'r')
		{
			cout << "removing hydrogens..." << endl;
			RemoveHydrogens(); UpdateAllViewWindows();
		}
		break;
		
		case SIGNAL_MM1_BUILDER:
		cout << "build amino/nucleic acid sequences ? (a/n) "; cin >> input;
		
		if (input == 'a')
		{
			cout << "give the sequence using symbols ARNDCQEGHILKMFPSTWYV ???"; cin >> seq;
			cout << "please select the conformation : H = helix, S = sheet ???"; cin >> input;
			mm1_mdl::amino_builder.Build(this, seq, input == 'h' || input == 'H' ? aab_ah : aab_bs);
			UpdateAllViewWindows();
		}
		
		if (input == 'n')
		{
			cout << "give the sequence using symbols AGTC and aguc ???"; cin >> seq;
			mm1_mdl::nucleic_builder.Build(this, seq, nab);
			UpdateAllViewWindows();
		}
		break;
		
/*		case SIGNAL_POPUP_MM1_RENDER_CMODE_STATE:
		win->colormode = & mm1_gui::cm_state; UpdateAllViewWindows();
		break;
		
		case SIGNAL_POPUP_MM1_RENDER_CMODE_ELEMENT:
		win->colormode = & mm1_gui::cm_element; UpdateAllViewWindows();
		break;		*/
		
		case SIGNAL_MM1_RMODE_WIREFRAME:
		win->render = RENDER_WIREFRAME; UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_RMODE_BALLANDSTICK:
		win->render = RENDER_BALL_AND_STICK; UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_RMODE_VANDERWAALS:
		win->render = RENDER_VAN_DER_WAALS; UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_RMODE_CYLINDERS:
		win->render = RENDER_CYLINDERS; UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_RMODE_NOTHING:
		win->render = RENDER_NOTHING; UpdateAllViewWindows();
		break;
		
/*		case SIGNAL_POPUP_MM1_RENDER_LABEL_ELEMENT:
		win->label = LABEL_ELEMENT; UpdateAllViewWindows();
		break;
		
		case SIGNAL_POPUP_MM1_RENDER_LABEL_BONDTYPE:
		win->label = LABEL_BONDTYPE; UpdateAllViewWindows();
		break;
		
		case SIGNAL_POPUP_MM1_RENDER_LABEL_NOTHING:
		win->label = LABEL_NOTHING; UpdateAllViewWindows();
		break;		*/
		
// geomopt + moldyn are here, but are done in a foolish way, just copied from the above...
// geomopt + moldyn are here, but are done in a foolish way, just copied from the above...
// geomopt + moldyn are here, but are done in a foolish way, just copied from the above...
		
		case SIGNAL_MM1_COMPUTE_GEOMOPT:
		cout << "how many cg-steps (500 might be Ok) ??? "; cin >> n1;
		
		eng = new mm1_eng_exp9(* this, 1, 0);		// DEFAULT ENGINE!!!
//		eng = new mm1_eng_exp1(* this, 1, 0);
//		GatherGroups(); eng = new mm1_eng_exp1_mim(* this, 1, 0);
		
		CopyCRD(this, eng, 0); opt = new mm1_geomopt(eng, 100, 0.01);
		
		for (n2 = 0;n2 < n1;n2++)
		{
			opt->TakeCGStep(conjugate_gradient::Newton2An);
			cout << n2 << " " << opt->optval << " " << opt->optstp << " " << endl;
			
			if (!(n2 % 10))
			{
				CopyCRD(eng, this, 0); CenterCRDSet(0);
				UpdateAllViewWindows(true);
			}
		}
		delete opt; delete eng;
		break;
		
		case SIGNAL_MM1_COMPUTE_MOLDYN:
		cout << "how many md-steps for simulation (10000 might be Ok) ??? "; cin >> n1;
		
		cout << "there is " << START << " heating steps, total steps = " << (n1 + START) << endl;
		cout << "the trajectory file will contain " << (n1 / 100) << " frames!!!" << endl;
		
		cout << "time-step length [fs] (0.5 or less might be Ok) ??? "; cin >> tmp1;
		cout << "traj-file name ??? "; cin >> buffer;
		
		eng = new mm1_eng_exp9(* this, 1, 0);		// DEFAULT ENGINE!!!
//		eng = new mm1_eng_exp1(* this, 1, 0);
//		GatherGroups(); eng = zzz = yyy = new mm1_eng_exp1_mim(* this, 1, 0);
		
		CopyCRD(this, eng, 0); dyn = new mm1_moldyn(eng, 300.0, tmp1);
		
		ofile.open(buffer, ios::out | ios::binary);
		for (n2 = 0;n2 < n1 + START;n2++)
		{
			if (!(n2 % 1000) && zzz != NULL) zzz->CheckLocations();
			if (!(n2 % 100) && yyy != NULL) yyy->update = true;
			
			dyn->TakeMDStep();
			if (!(n2 % 50))
			{
				cout << n2 << " T = " << dyn->ConvEKinTemp(dyn->kin);
				cout << " pot = " << dyn->pot << " tot = " << (dyn->kin + dyn->pot) << endl;
			}
			
			if (n2 < START)
			{
				if (!(n2 % CFRQ))
				{
					control[ccnt++] = dyn->kin;
				}
				
				if (ccnt == CSIZE)
				{
					f64 sum = 0.0;
					for (i32s n9 = 0;n9 < ccnt;n9++)
					{
						sum += control[n9];
					}
					
					sum /= (f64) ccnt; ccnt = 0; f64 rtemp = (f64) (n2 + CFRQ) / (f64) START;
					cout << "average = " << sum << " kJ/mol = " << dyn->ConvEKinTemp(sum) << " K." << endl;
					dyn->SetEKin(dyn->kin + (dyn->ConvTempEKin(dyn->temp * rtemp) - sum));
				}
			}
			else if (!(n2 % 100))
			{
				CopyCRD(eng, this, 0);
				
				ofile.write((char *) & dyn->kin, sizeof(dyn->kin));
				ofile.write((char *) & dyn->pot, sizeof(dyn->pot));
				
				for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
				{
					for (i32s t4 = 0;t4 < 3;t4++)
					{
						f64 t1a = (* it1).crd_vector[0][t4];
						ofile.write((char *) & t1a, sizeof(t1a));
					}
				}
			}
			
			if (!(n2 % 100))
			{
				CopyCRD(eng, this, 0);
				UpdateAllViewWindows(true);
			}
		}
		
		delete dyn; delete eng;
		UpdateAccumValues();
		break;
		
		case SIGNAL_MM1_OBJECTS_RIBBON:
		mm1_mdl::amino_builder.Identify(this);
		DefineSecondaryStructure(this);
		
		AddObject(new mm1_ribbon(this, win->colormode, 0, 4));		// min. order is 2!!!
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_OBJECTS_EPLANE:
		cout << "give dimension of the plane [nm] (1.0 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) mm1_GetESPValue; cpp.cf = GetRGBRange2;
		cpp.value = 4.1868 * 33.20716; cpp.descr = "ESP-";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_OBJECTS_ESURFACE:	tmp1 = 4.1868 * 33.20716;
		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) mm1_GetESPValue; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetRedColor; csp.svalue = +tmp1; csp.descr = "pos ESP-";
		AddObject(new color_surface(ol_static(), csp));
		
		csp.cf = GetBlueColor; csp.svalue = -tmp1; csp.descr = "neg ESP-";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_OBJECTS_VDWSURFACE:	tmp1 = 4.1868 * 33.20716 / 2.0;
		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false; csp.cf = GetRGBRange2;
		csp.vf1 = (ValueFunction *) mm1_GetSASValue; csp.vf2 = (ValueFunction *) mm1_GetESPValue;
		csp.svalue = 1.0; csp.cvalue = tmp1; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		csp.descr = "ESP-colored VDW-";
		
		AddObject(new color_surface(ol_static(), csp));
		UpdateAllViewWindows();
		break;
		
		default:
		cout << "could not handle signal 0x" << hex << signal << dec << endl;
	}
}

#endif	// TARGET3

////////////////////////////////////////////////////////////////////////////////////////////////////

void mm1_cm_element::GetColor(const void * p1, fGL_a3 & p2)
{
	mm1_atom * ref = (mm1_atom *) p1;
	const fGL * color = ref->el.GetColor();
	for (i32s n1 = 0;n1 < 3;n1++) p2[n1] = color[n1];
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void mm1_cm_state::GetColor(const void * p1, fGL_a3 & p2)
{
//	mm1_atom * ref = (mm1_atom *) p1;
//	mm1_mdl * mdl = ref->mdl;		// CURRENTLY BROKEN...
	
	p2[0] = 0.0; p2[1] = 0.0; p2[2] = 1.0;		// loop
	
/*	if (mdl->chn_info == NULL) return;
	vector<mm1_chn_info *> & ci = * mdl->chn_info;
	
	if (ref->id[1] < 0 || ref->id[2] < 0) return;
	if (ci[ref->id[1]]->state == NULL) return;
	
	char state = ci[ref->id[1]]->state[ref->id[2]];
	
	switch (state)
	{
		case '4':
		p2[0] = 1.0; p2[1] = 0.0; p2[2] = 0.0;		// helix
		return;
		
		case 'S':
		p2[0] = 0.0; p2[1] = 1.0; p2[2] = 0.0;		// strand
		return;
	}*/
}

////////////////////////////////////////////////////////////////////////////////////////////////////

fGL mm1_GetESPValue(fGL * crd, mm1_gui * ref, fGL * gdt)
{
	fGL sum1 = 0.0;
	if (gdt != NULL) gdt[0] = gdt[1] = gdt[2] = 0.0;
	for (iter_mm1al it1 = ref->atom_list.begin();it1 != ref->atom_list.end();it1++)
	{
		fGL tmp1[3]; fGL sum2 = 0.0;
		for (i32s n1 = 0;n1 < 3;n1++)
		{
			tmp1[n1] = crd[n1] - (* it1).crd_vector[0][n1];
			sum2 += tmp1[n1] * tmp1[n1];
		}
		
		if (sum2 == 0.0) return +1.0e+35;	// numeric_limits<fGL>::max() !??!?!
		
		// do we have a correct constant here??? I think so, if we define
		// electrostatic potential as potential energy of a unit positive charge.
		
		sum2 = sqrt(sum2);
		fGL tmp2 = 4.1868 * 33.20716 * (* it1).charge / sum2;
		sum1 += tmp2;
		
		if (gdt != NULL)	// sign ??? constant ???
		{
			for (i32s n1 = 0;n1 < 3;n1++)
			{
				fGL tmp3 = tmp1[n1] / sum2;
				fGL tmp4 = tmp3 * tmp2 / sum2;
				gdt[n1] += tmp4;
			}
		}
	}
	
	return sum1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// f = sum[(r/a)^-3] = sum[a^3 * r^-3]
// df/dr = -3 * sum[a^3 * r^-4]

fGL mm1_GetSASValue(fGL * crd, mm1_gui * ref, fGL * gdt)
{
	fGL sum1 = 0.0; if (gdt != NULL) gdt[0] = gdt[1] = gdt[2] = 0.0;
	for (iter_mm1al it1 = ref->atom_list.begin();it1 != ref->atom_list.end();it1++)
	{
		fGL tmp1[3]; fGL sum2 = 0.0;
		for (i32s n1 = 0;n1 < 3;n1++)
		{
			tmp1[n1] = crd[n1] - (* it1).crd_vector[0][n1];
			sum2 += tmp1[n1] * tmp1[n1];
		}
		
		if (sum2 == 0.0) return +1.0e+35;	// numeric_limits<fGL>::max() ?!?!!?
		
		// calculate r to sum2 and f to tmp3...
		// calculate r to sum2 and f to tmp3...
		// calculate r to sum2 and f to tmp3...
		
		sum2 = sqrt(sum2);
		fGL tmp2 = sum2 / (* it1).el.GetVDWRadius();
		fGL qqq = tmp2 * tmp2 * tmp2 * tmp2;
		fGL tmp3 = 1.0 / (qqq * qqq * qqq);
		sum1 += tmp3;
		
		// calculate dr/dx to tmp4 and components to tmp5.
		// does tmp5 have sign/constant errors !?!?!?!
		
		if (gdt != NULL)
		{
			for (i32s n1 = 0;n1 < 3;n1++)
			{
				fGL tmp4 = tmp1[n1] / sum2;
				fGL tmp5 = tmp4 * tmp3 / tmp2;
				gdt[n1] += tmp5;
			}
		}
	}
	
	return sum1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI

// eof
