// MM1GUI.H : a graphical user interface for mm1-models.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef MM1GUI_H
#define MM1GUI_H

#ifdef ENABLE_GUI

class mm1_gui;

class mm1_cm_element;		// mm1_atom *
class mm1_cm_state;		// mm1_atom *

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "guibase.h"
#include "mm1mdl.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET3
#include <gnome.h>

#define SIGNAL_MM1_FILE_SAVEAS			0x21008112
#define SIGNAL_MM1_FILE_CLOSE			0x21008113

#define SIGNAL_MM1_SETTINGS_ELEMENT		0x2100B010
#define SIGNAL_MM1_SETTINGS_BONDTYPE		0x2100B011

#define SIGNAL_MM1_RMODE_WIREFRAME		0x21009000
#define SIGNAL_MM1_RMODE_BALLANDSTICK		0x21009001
#define SIGNAL_MM1_RMODE_VANDERWAALS		0x21009002
#define SIGNAL_MM1_RMODE_CYLINDERS		0x21009003
#define SIGNAL_MM1_RMODE_NOTHING		0x21009004

#define SIGNAL_MM1_OBJECTS_RIBBON		0x2100A000
#define SIGNAL_MM1_OBJECTS_EPLANE		0x2100A001
#define SIGNAL_MM1_OBJECTS_ESURFACE		0x2100A002
#define SIGNAL_MM1_OBJECTS_VDWSURFACE		0x2100A003

#define SIGNAL_MM1_COMPUTE_GEOMOPT		0x2100B000
#define SIGNAL_MM1_COMPUTE_MOLDYN		0x2100B001

#define SIGNAL_MM1_HYDROGENS			0x2100C000
#define SIGNAL_MM1_BUILDER			0x2100C001

#endif	// TARGET3

class mm1_gui : public bs_gui, public mm1_mdl
{
	public:
	
	static mm1_cm_element cm_element;
	static mm1_cm_state cm_state;
	
	mm1_atom * draw_data[2];
	
	bool draw_box;
	bool draw_images;
	
	public:
	
	mm1_gui(window_factory &, ostream *);
	~mm1_gui(void);
	
	const char * GetType(void);			// virtual
	color_mode * GetDefaultColorMode(void);		// virtual
	
	void SelectAll(void);				// virtual
	void InvertSelection(void);			// virtual
	
	bool TestAtom(mm1_atom *, rmode);
	bool TestBond(mm1_bond *, rmode);
	
	void SetColor(color_mode *, mm1_atom *);
	
	void InitGL(void);				// virtual
	void Render(view_window *, rmode);		// virtual
	
	void RenderScene(view_window *, rmode, bool);
	
	void Center(transformer *);			// virtual
	void Transform(transformer *);			// virtual
	
	void DrawEvent(view_window *, vector<iGLu> &);		// virtual
	void EraseEvent(view_window *, vector<iGLu> &);		// virtual
	void SelectEvent(view_window *, vector<iGLu> &);	// virtual
	
#ifdef TARGET2

	void KeyEvent(view_window *, char, int, int);		// virtual
	
#endif	// TARGET2

#ifdef TARGET3

	static GnomeUIInfo file_submenu[];
	static GnomeUIInfo select_submenu[];
	static GnomeUIInfo settings_submenu[];
	
	static GnomeUIInfo views_submenu[];
	static GnomeUIInfo viewscreate_submenu[];
	static GnomeUIInfo lights_submenu[];
	static GnomeUIInfo rendermode_submenu[];
	static GnomeUIInfo projection_submenu[];
	static GnomeUIInfo render_submenu[];
	
	static GnomeUIInfo objects_submenu[];
	static GnomeUIInfo compute_submenu[];
	
	static GnomeUIInfo popupmenu_info[];
	
	void PopupEvent(view_window *, int);			// virtual
	
#endif	// TARGET3

};

////////////////////////////////////////////////////////////////////////////////////////////////////

class mm1_cm_element : public color_mode
{
	protected:
	
	public:
	
	mm1_cm_element(void) : color_mode() { }
	~mm1_cm_element(void) { }
	
	void GetColor(const void *, fGL_a3 &);
};

class mm1_cm_state : public color_mode
{
	protected:
	
	public:
	
	mm1_cm_state(void) : color_mode() { }
	~mm1_cm_state(void) { }
	
	void GetColor(const void *, fGL_a3 &);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

fGL mm1_GetESPValue(fGL *, mm1_gui *, fGL *);
fGL mm1_GetSASValue(fGL *, mm1_gui *, fGL *);

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI
#endif	// MM1GUI_H

// eof
