// PLANE.H : a class for drawing colored 3D-planes.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef PLANE_H
#define PLANE_H

#ifdef ENABLE_GUI

struct cp_param;
class color_plane;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "viewing.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

struct cp_param
{
	i32s np; fGL dim;
	bool transparent;
	
	bs_gui * gui;
	
	void * ref;
	ValueFunction * vf;
	ColorFunction * cf;
	
	fGL value;
	const char * descr;
};

class color_plane : public smart_object
{
	public:
	
	bs_gui * gui;
	i32s np; fGL dim;
	iGLu list_id;
	
	void * ref;
	ValueFunction * GetValue;
	ColorFunction * GetColor;
	
	fGL value;
	
	fGL * comp1; fGL * comp2;
	fGL * comp3; fGL * dist;
	
	v3d<fGL> xdir;
	
	char name[256];
	
	public:
	
	color_plane(const object_location &, cp_param &);
	~color_plane(void);
	
	void Update(void);
	
	void SetDimension(fGL);
	void GetCRD(i32s *, fGL *);
	
	const char * GetObjectName(void) { return name; }	// virtual
	
	bool BeginTransformation(void) { return false; }	// virtual
	bool EndTransformation(void) { return false; }		// virtual
	
	void OrbitObject(const fGL *, const camera &);		// virtual
	void RotateObject(const fGL *, const camera &);		// virtual
	
	void TranslateObject(const fGL *, const obj_loc_data *);	// virtual
	
	void Render(void);		// virtual
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI
#endif	// PLANE_H

// eof
