// QM1GUI.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "qm1gui.h"

#ifdef ENABLE_GUI
#include "plane.h"
#include "surface.h"

#include "color.h"
#include "window.h"

//#ifdef TARGET3
//#include "t3main.h"
//#endif	// TARGET3

////////////////////////////////////////////////////////////////////////////////////////////////////

qm1_gui::qm1_gui(window_factory & p1, ostream * p2) : bs_gui(p1, p2), qm1_mdl(p2), bs_mdl_simple(p2)
{
}

qm1_gui::~qm1_gui(void)
{
}

const char * qm1_gui::GetType(void)
{
	return "QM, atomic calculations";
}

color_mode * qm1_gui::GetDefaultColorMode(void)
{
	return NULL;
}

void qm1_gui::SelectAll(void)
{
}

void qm1_gui::InvertSelection(void)
{
}

// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!

void qm1_gui::InitGL(void)
{
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glDepthFunc(GL_LESS); glEnable(GL_DEPTH_TEST);
	
	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 64);
	const fGL specular_reflectance[4] = { 0.9, 0.9, 0.9, 1.0 };
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular_reflectance);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	
	const fGL ambient_intensity[4] = { 0.2, 0.2, 0.2, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_intensity);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, true);
	
	glFogi(GL_FOG_MODE, GL_EXP); glFogf(GL_FOG_DENSITY, 0.15);
	const fGL fog_color[4] = { 0.0, 0.0, 0.0, 0.0 };
	glFogfv(GL_FOG_COLOR, fog_color);
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glSelectBuffer(SB_SIZE, select_buffer);
}

void qm1_gui::Render(view_window * win, rmode rm)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	if (win->enable_fog) glEnable(GL_FOG);
	
	glPointSize(3.0); glLineWidth(1.0);
	
	glBegin(GL_POINTS);
	glColor3fv(el.GetColor());
	glVertex3f(0.0, 0.0, 0.0);
	glEnd();
	
	if (rm != Transform2) win->cam->RenderObjects(win);
	if (win->enable_fog) glDisable(GL_FOG);
}

void qm1_gui::Center(transformer *)
{
}

void qm1_gui::Transform(transformer *)
{
}

void qm1_gui::DrawEvent(view_window *, vector<iGLu> &)
{
}

void qm1_gui::EraseEvent(view_window *, vector<iGLu> &)
{
}

void qm1_gui::SelectEvent(view_window *, vector<iGLu> &)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET2

void qm1_gui::KeyEvent(view_window * win, char key, int, int)
{
//	fGL tmp1; fGL tmp2;
//	i32s sz[3] = { 50, 50, 50 };
//	fGL dm[3] = { 0.55, 0.50, 0.45 };
	
	switch (key)
	{
		case 'h':
		cout << "+ -       -> change the element" << endl;
		break;
		
		case '+': ++el;
		cout << "element = " << el.GetAtomicNumber();
		cout << " " << el.GetSymbol() << endl;
		break;
		
		case '-': --el;
		cout << "element = " << el.GetAtomicNumber();
		cout << " " << el.GetSymbol() << endl;
		break;
	}
}

#endif	// TARGET2

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef TARGET3

void qm1_gui::PopupEvent(view_window * win, int signal)
{
	switch (signal)
	{
	}
}

#endif	// TARGET3

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI

// eof
