// T3MAIN.CPP

// Copyright (C) 1999 Tommi Hassinen, Ville Heikkil.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "t3main.h"

#include "notice.h"
//#include "t3dialog.h"

#include <strstream>

////////////////////////////////////////////////////////////////////////////////////////////////////

GnomeUIInfo gnome_app_gt3::file_new_submenu[] =
{
	GNOMEUIINFO_MENU_NEW_ITEM("QM2", NULL,		// "Quantum Mechanics project, model of a molecular system." ???
		gnome_app_gt3::MenuItemHandler, SIGNAL_APP_FILE_NEW_QM2),
	
	GNOMEUIINFO_MENU_NEW_ITEM("MM1", NULL,		// "Molecular Mechanics project, an \"all atoms\"-model." ???
		gnome_app_gt3::MenuItemHandler, SIGNAL_APP_FILE_NEW_MM1),
	
	GNOMEUIINFO_MENU_NEW_ITEM("MM2", NULL,		// "Molecular Mechanics project, a reduced protein model." ???
		gnome_app_gt3::MenuItemHandler, SIGNAL_APP_FILE_NEW_MM2),
	
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_app_gt3::file_submenu[] =
{
	GNOMEUIINFO_SUBTREE("_New...", file_new_submenu),
	GNOMEUIINFO_MENU_OPEN_ITEM(gnome_app_gt3::MenuItemHandler, NULL),
	GNOMEUIINFO_MENU_EXIT_ITEM(gnome_app_gt3::MenuItemHandler, SIGNAL_APP_FILE_EXIT),
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_app_gt3::help_submenu[] =
{
	GNOMEUIINFO_MENU_ABOUT_ITEM(gnome_app_gt3::MenuItemHandler, SIGNAL_APP_HELP_ABOUT),
	GNOMEUIINFO_END
};

GnomeUIInfo gnome_app_gt3::menubar_info[] =
{
	GNOMEUIINFO_SUBTREE("_File", file_submenu),
	GNOMEUIINFO_SUBTREE("_Help", help_submenu),
	GNOMEUIINFO_END
};

////////////////////////////////////////////////////////////////////////////////////////////////////

GnomeUIInfo gnome_app_gt3::toolbar_info[] =
{
	{
		GNOME_APP_UI_TOGGLEITEM, "Draw",
		"This tool is used to add atoms and bonds to the model.",
		gnome_app_gt3::ToolButtonHandler, & window::tool_draw, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},

	{
		GNOME_APP_UI_TOGGLEITEM, "Erase",
		"This tool is used to remove atoms and bonds from the model.",
		gnome_app_gt3::ToolButtonHandler, & window::tool_erase, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},
		
	{
		GNOME_APP_UI_TOGGLEITEM, "Select",
		"This tool is used to make selections in the model, and also to select objects.",
		gnome_app_gt3::ToolButtonHandler, & window::tool_select, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},
	
	{
		GNOME_APP_UI_TOGGLEITEM, "Zoom",
		"This tool is used to ",
		gnome_app_gt3::ToolButtonHandler, & window::tool_zoom, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},
	
	{
		GNOME_APP_UI_TOGGLEITEM, "Translate XY",
		"This tool is used to translate the camera or a selected object in XY-direction.",
		gnome_app_gt3::ToolButtonHandler, & window::tool_translate_xy, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},
	
	{
		GNOME_APP_UI_TOGGLEITEM, "Translate Z",
		"This tool is used to ",
		gnome_app_gt3::ToolButtonHandler, & window::tool_translate_z, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},
	
	{
		GNOME_APP_UI_TOGGLEITEM, "Orbit XY",
		"This tool is used to ",
		gnome_app_gt3::ToolButtonHandler, & window::tool_orbit_xy, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},
	
	{
		GNOME_APP_UI_TOGGLEITEM, "Orbit Z",
		"This tool is used to ",
		gnome_app_gt3::ToolButtonHandler, & window::tool_orbit_z, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},
	
	{
		GNOME_APP_UI_TOGGLEITEM, "Rotate XY",
		"This tool is used to ",
		gnome_app_gt3::ToolButtonHandler, & window::tool_rotate_xy, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},
	
	{
		GNOME_APP_UI_TOGGLEITEM, "Rotate Z",
		"This tool is used to ",
		gnome_app_gt3::ToolButtonHandler, & window::tool_rotate_z, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_OPEN
	},
	
	GNOMEUIINFO_END
};

////////////////////////////////////////////////////////////////////////////////////////////////////

GtkWidget * gnome_app_gt3::main_window = NULL;
GtkWidget * gnome_app_gt3::current_tool_widget = NULL;

gnome_app_gt3 * gnome_app_gt3::instance = NULL;

gnome_app_gt3::gnome_app_gt3(void)
{
	char title[256]; ostrstream str(title, sizeof(title));
	str << get_copyright_notice_line(0) << " - version " << VERSION << ends;
	
	main_window = gnome_app_new("ghemical", title);
	
	gtk_window_set_default_size(GTK_WINDOW(main_window), gdk_screen_width() - 100, 10);
	gtk_widget_set_uposition(main_window, 100 - 15, 5);
	
	// setup handlers...
	
	gtk_signal_connect(GTK_OBJECT(main_window), "delete_event", GTK_SIGNAL_FUNC(DeleteEventHandler), NULL);
	gtk_signal_connect(GTK_OBJECT(main_window), "destroy", GTK_SIGNAL_FUNC(DestroyHandler), NULL);
	
	// setup widgets...
	
	gnome_app_create_menus(GNOME_APP(main_window), menubar_info);
	gnome_app_create_toolbar(GNOME_APP(main_window), toolbar_info);
	
	tool_draw_widget = toolbar_info[0].widget;
	tool_erase_widget = toolbar_info[1].widget;
	tool_select_widget = toolbar_info[2].widget;
	tool_zoom_widget = toolbar_info[3].widget;
	tool_translate_xy_widget = toolbar_info[4].widget;
	tool_translate_z_widget = toolbar_info[5].widget;
	tool_orbit_xy_widget = toolbar_info[6].widget;
	tool_orbit_z_widget = toolbar_info[7].widget;
	tool_rotate_xy_widget = toolbar_info[8].widget;
	tool_rotate_z_widget = toolbar_info[9].widget;
	
	// orbit_xy is the default tool, set it active...
	
	current_tool_widget = tool_orbit_xy_widget;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(current_tool_widget), TRUE);
	
	// ready...
	
	gtk_widget_show(main_window);
}

gnome_app_gt3::~gnome_app_gt3(void)
{
}

gnome_app_gt3 * gnome_app_gt3::GetInstance(void)
{
	if (instance != NULL) return instance;
	else return (instance = new gnome_app_gt3());
}

void gnome_app_gt3::HelpAbout(void)
{
	GtkWidget * aboutdialog;
	
	const char * txt;
	txt = get_copyright_notice_line(0); char * buffer1 = new char[strlen(txt)]; strcpy(buffer1, txt);
	txt = get_copyright_notice_line(5); char * buffer2 = new char[strlen(txt)]; strcpy(buffer2, txt);
	
	const gchar * authors[] =
	{
		"Tommi Hassinen",
		NULL
	};
	
	char intro[256];
	ostrstream intro_str(intro, sizeof(intro));
	
	intro_str << "Ghemical is a molecular modelling package released under the GNU GPL." << endl;
	intro_str << get_copyright_notice_line(3) << ends;
	
	aboutdialog = gnome_about_new(buffer1, VERSION, buffer2, authors, intro, NULL);
	
	delete[] buffer1; delete[] buffer2;
	gtk_widget_show(aboutdialog);
}

gboolean gnome_app_gt3::CheckExit(void)
{
	GtkWidget * msgbox;
	
	msgbox = gnome_message_box_new("Are you sure you want to quit ?",
		GNOME_MESSAGE_BOX_QUESTION, GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
	
	return (gnome_dialog_run_and_close(GNOME_DIALOG(msgbox)) == 0);
}

void gnome_app_gt3::MenuItemHandler(GtkWidget *, gpointer signal)
{
	bs_gui * gui = NULL;
	
	switch ((unsigned int) signal)
	{
		case SIGNAL_APP_FILE_NEW_QM2:
		gui = new qm2_gui(* gtkgl_factory::GetInstance(), & cout);
		gui->SetDefaultProjectFileName(); gui->AddCamera(2.0);
		break;
		
		case SIGNAL_APP_FILE_NEW_MM1:
		gui = new mm1_gui(* gtkgl_factory::GetInstance(), & cout);
		gui->SetDefaultProjectFileName(); gui->AddCamera(2.0);
		break;
		
		case SIGNAL_APP_FILE_EXIT:
		if (GetInstance()->CheckExit()) gtk_widget_destroy(main_window);
		break;
		
		case SIGNAL_APP_HELP_ABOUT:
		GetInstance()->HelpAbout();
		break;
	}
}

void gnome_app_gt3::ToolButtonHandler(GtkWidget * widget, gpointer tool)
{
	gtk_signal_handler_block_by_func(GTK_OBJECT(current_tool_widget),
		(GtkSignalFunc) gnome_app_gt3::ToolButtonHandler, window::current_tool);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(current_tool_widget), FALSE);
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(current_tool_widget),
		(GtkSignalFunc) gnome_app_gt3::ToolButtonHandler, window::current_tool);
	
	current_tool_widget = widget;
	window::current_tool = (mouse_tool *) tool;
	
	gtk_signal_handler_block_by_func(GTK_OBJECT(current_tool_widget),
		(GtkSignalFunc) gnome_app_gt3::ToolButtonHandler, window::current_tool);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(current_tool_widget), TRUE);
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(current_tool_widget),
		(GtkSignalFunc) gnome_app_gt3::ToolButtonHandler, window::current_tool);
}

gboolean gnome_app_gt3::DeleteEventHandler(GtkWidget *, GdkEvent *, gpointer)
{
	return !(GetInstance()->CheckExit());
}

void gnome_app_gt3::DestroyHandler(GtkWidget *, gpointer)
{
	gtk_main_quit();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

singleton_cleaner<gnome_app_gt3> app_cleaner;

int main(int argc, char ** argv)
{
	gnome_init("ghemical", VERSION, argc, argv);
	app_cleaner.SetInstance(gnome_app_gt3::GetInstance());
	gtk_main(); gtk_exit(0);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
