// T3WINDOW.CPP

// Copyright (C) 1999 Tommi Hassinen, Ville Heikkil.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "t3window.h"

#include "guibase.h"
#include "utility.h"

#include "mm1gui.h"	// temporary !?!?!?!?

#include <algorithm>
#include <gnome.h>

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<gtkgl_window *> gtkgl_window::gtkgl_vector;

gtkgl_window::gtkgl_window(bs_gui * p1) : window(p1)
{
	gtkwin = NULL;
	glarea = NULL;
}

gtkgl_window::~gtkgl_window(void)
{
	vector<gtkgl_window *>::iterator it1;
	it1 = find(gtkgl_vector.begin(), gtkgl_vector.end(), this);
	
	if (it1 != gtkgl_vector.end())
	{
		gtkgl_vector.erase(it1);
		gtk_widget_destroy(gtkwin);	// what about the glarea !?!?!?!?
	}
}

void gtkgl_window::CreateWindow(void)
{
	gtkgl_vector.push_back(this);
	
	gtkwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_quit_add_destroy(1, GTK_OBJECT(gtkwin));
	
	int attrlst[] =
	{
		GDK_GL_RGBA,
		GDK_GL_RED_SIZE, 1,
		GDK_GL_GREEN_SIZE, 1,
		GDK_GL_BLUE_SIZE, 1,
		GDK_GL_DEPTH_SIZE, 1,
		GDK_GL_DOUBLEBUFFER,
		GDK_GL_NONE
	};
	
	glarea = gtk_gl_area_new(attrlst);
	
	gtk_widget_set_usize(GTK_WIDGET(glarea), 400, 400);
//	gtk_drawing_area_size(GTK_DRAWING_AREA(glarea), 400, 400);	// no effect?!?!?!?

	int events = GDK_EXPOSURE_MASK;
	events |= GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK;
	events |= GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK;
	
	gtk_widget_set_events(GTK_WIDGET(glarea), events);
	
	gtk_signal_connect(GTK_OBJECT(glarea),
		"expose_event", GTK_SIGNAL_FUNC(ExposeHandler), NULL);
	
	gtk_signal_connect(GTK_OBJECT(glarea),
		"motion_notify_event", GTK_SIGNAL_FUNC(MotionNotifyHandler), NULL);
	
	gtk_signal_connect(GTK_OBJECT(glarea),
		"button_press_event", GTK_SIGNAL_FUNC(ButtonHandler), NULL);
	
	gtk_signal_connect(GTK_OBJECT(glarea),
		"button_release_event", GTK_SIGNAL_FUNC(ButtonHandler), NULL);
	
	gtk_signal_connect(GTK_OBJECT(glarea),
		"destroy", GTK_SIGNAL_FUNC(DestroyHandler), NULL);
	
	gtk_signal_connect(GTK_OBJECT(glarea),
		"configure_event", GTK_SIGNAL_FUNC(ConfigureHandler), NULL);
	
	gtk_container_add(GTK_CONTAINER(gtkwin), glarea);
	gtk_container_set_border_width(GTK_CONTAINER(gtkwin), 1);
	
	gtk_widget_show(GTK_WIDGET(glarea));
	gtk_widget_show(GTK_WIDGET(gtkwin));
	
	SetCurrent(); gui->InitGL();
}

void gtkgl_window::SetTitle(const char * p1)
{
	gtk_window_set_title(GTK_WINDOW(gtkwin), p1);
}

void gtkgl_window::SetCurrent(void)
{
	bool failed = !gtk_gl_area_make_current(GTK_GL_AREA(glarea));
	if (failed) cout << "SetCurrent(): can't make_current!!!" << endl;
}

void gtkgl_window::Update(bool directly)
{
	if (directly) ExposeEvent();
	else ExposeEvent();
	
	// here we update always directly. should emit an update request for indirect rendering!?!?!?
	// here we update always directly. should emit an update request for indirect rendering!?!?!?
	// here we update always directly. should emit an update request for indirect rendering!?!?!?
}

gtkgl_window * gtkgl_window::GetWindow(GtkWidget * glarea_widget)
{
	vector<gtkgl_window *>::iterator it1 = gtkgl_vector.begin();
	while (it1 != gtkgl_vector.end())
	{
		vector<gtkgl_window *>::iterator it2 = it1++;
		if ((* it2)->glarea == glarea_widget) return (* it2);
	}
	
	// return NULL if the search failed...
	// return NULL if the search failed...
	// return NULL if the search failed...
	
	return NULL;
}

gint gtkgl_window::ExposeHandler(GtkWidget * widget, GdkEventExpose *)
{
	gtkgl_window * win = GetWindow(widget);
	if (!win) cout << "Unknown ID in ExposeHandler !!!" << endl;
	else win->ExposeEvent();
	
	return FALSE;
}

gint gtkgl_window::ButtonHandler(GtkWidget * widget, GdkEventButton * eb)
{
	gtkgl_window * win = GetWindow(widget);
	if (!win) cout << "Unknown ID in ButtonHandler !!!" << endl;
	else
	{
		mouse_tool::button tmpb; i32s tmps1;
		switch (eb->button)
		{
			case 1:
			tmpb = mouse_tool::Left;
			tmps1 = GDK_BUTTON1_MASK;
			break;
			
			case 3:
			tmpb = mouse_tool::Right;
			tmps1 = GDK_BUTTON3_MASK;
			break;
			
			default:
			tmpb = mouse_tool::Middle;
			tmps1 = GDK_BUTTON2_MASK;
		}
		
		mouse_tool::state tmps2 = (eb->state & tmps1) ? mouse_tool::Up : mouse_tool::Down;
		
		view_window * view_win = dynamic_cast<view_window *>(win);
		
		if (tmps2 == mouse_tool::Down)
		{
			if (button == mouse_tool::None)
			{
				if (tmpb == mouse_tool::Right)
				{
					// the popup menu is created here. pointer to the glarea widget
					// is given as "user_data", and it is also passed to the popup
					// hander callback function (instead of the original value).
					
					gnome_popup_menu_do_popup(GetWindow(widget)->gui->popupmenu,
						NULL, NULL, NULL, win->glarea);
						
					return TRUE;
				}
				
				button = tmpb;
				
				shift_down = (eb->state & GDK_SHIFT_MASK) ? true : false;
				ctrl_down = (eb->state & GDK_CONTROL_MASK) ? true : false;
				
				state = mouse_tool::Down;
				
				if (view_win != NULL) current_tool->ButtonEvent(view_win, eb->x, eb->y);
			}
		}
		else
		{
			if (button == mouse_tool::Left && tmpb != mouse_tool::Left) return TRUE;
			if (button == mouse_tool::Middle && tmpb != mouse_tool::Middle) return TRUE;
			if (button == mouse_tool::Right && tmpb != mouse_tool::Right) return TRUE;
			
			state = mouse_tool::Up;
			
			if (view_win != NULL) current_tool->ButtonEvent(view_win, eb->x, eb->y);
			
			button = mouse_tool::None;
		}
	}
	
	return TRUE;
}

void gtkgl_window::DestroyHandler(GtkObject * object)
{
	GtkWidget * widget = GTK_WIDGET(object);
	
	gtkgl_window * win = GetWindow(widget);
	if (!win) cout << "Unknown ID in DestroyHandler !!!" << endl;
	view_window * view_win = dynamic_cast<view_window *>(win);
	
	if (view_win != NULL) view_win->gui->RemoveViewWindow(view_win);
}

gint gtkgl_window::ConfigureHandler(GtkWidget * widget, GdkEventConfigure *)
{
	gtkgl_window * win = GetWindow(widget);
	if (!win) cout << "Unknown ID in ConfigureHandler !!!" << endl;
	else
	{
		win->SetCurrent();
		win->SetSize(widget->allocation.width, widget->allocation.height);
	}
	
	return TRUE;
}

gint gtkgl_window::MotionNotifyHandler(GtkWidget * widget, GdkEventMotion * event)
{
	int x; int y; GdkModifierType state;
	if (event->is_hint) gdk_window_get_pointer(event->window, & x, & y, & state);
	else { x = (int) event->x; y = (int) event->y; state = (GdkModifierType) event->state; }
	
	if (button != mouse_tool::None)
	{
		gtkgl_window * win = GetWindow(widget);
		if (!win) cout << "Unknown ID in MotionNotifyHandler !!!" << endl;
		view_window * view_win = dynamic_cast<view_window *>(win);
		
		if (view_win != NULL) current_tool->MotionEvent(view_win, x, y);
	}
	
	return TRUE;
}

// the popup menu event handler. does not belong to the class, but is a friend function.
// the popup menu event handler. does not belong to the class, but is a friend function.
// the popup menu event handler. does not belong to the class, but is a friend function.

// since gnome_popup_menu_do_popup was used to create the menu, "signal" now contains
// not the original data, but the pointer to the glarea widget. using that we can find
// out which GUI should receive the signal. using "widget" we can find out which menu
// item was chosen...

void GtkGLPopupHandler(GtkWidget * widget, void * signal)
{
	gtkgl_window * win = gtkgl_window::GetWindow((GtkWidget *) signal);
	if (!win) cout << "Unknown ID in GtkGLPopupHandler !!!" << endl;
	
////////////////////////////////////////
////////////////////////////////////////

	// now we have to convert "widget" to the original "user_data".
	// there may be an easier way in GTK+/GNOME to do this, but haven't found it yet...
	
	i32s udata = NOT_DEFINED;
	
	if (widget == mm1_gui::file_submenu[0].widget) udata = SIGNAL_MM1_FILE_SAVEAS;
	if (widget == mm1_gui::file_submenu[1].widget) udata = SIGNAL_MM1_FILE_CLOSE;
	
	if (widget == mm1_gui::select_submenu[0].widget) udata = SIGNAL_GEN_SELECT_SELECTALL;
	if (widget == mm1_gui::select_submenu[1].widget) udata = SIGNAL_GEN_SELECT_INVERTSELECTION;
	
	if (widget == mm1_gui::settings_submenu[0].widget) udata = SIGNAL_MM1_SETTINGS_ELEMENT;
	if (widget == mm1_gui::settings_submenu[1].widget) udata = SIGNAL_MM1_SETTINGS_BONDTYPE;
	
	if (widget == mm1_gui::viewscreate_submenu[0].widget) udata = SIGNAL_GEN_VIEWS_NEWCAMERA;
	if (widget == mm1_gui::viewscreate_submenu[1].widget) udata = SIGNAL_GEN_VIEWS_NEWVIEW;
	
	if (widget == mm1_gui::views_submenu[1].widget) udata = SIGNAL_GEN_VIEWS_DELETEVIEW;
	
	if (widget == mm1_gui::lights_submenu[0].widget) udata = SIGNAL_GEN_LIGHTS_NEWLIGHT;
	
	if (widget == mm1_gui::rendermode_submenu[0].widget) udata = SIGNAL_MM1_RMODE_WIREFRAME;
	if (widget == mm1_gui::rendermode_submenu[1].widget) udata = SIGNAL_MM1_RMODE_BALLANDSTICK;
	if (widget == mm1_gui::rendermode_submenu[2].widget) udata = SIGNAL_MM1_RMODE_VANDERWAALS;
	if (widget == mm1_gui::rendermode_submenu[3].widget) udata = SIGNAL_MM1_RMODE_CYLINDERS;
	if (widget == mm1_gui::rendermode_submenu[4].widget) udata = SIGNAL_MM1_RMODE_NOTHING;
	
	if (widget == mm1_gui::projection_submenu[0].widget) udata = SIGNAL_GEN_PROJECTION_ORTHOGRAPHIC;
	if (widget == mm1_gui::projection_submenu[1].widget) udata = SIGNAL_GEN_PROJECTION_PERSPECTIVE;
	
	if (widget == mm1_gui::render_submenu[4].widget) udata = SIGNAL_GEN_RENDER_QUICKUPDATE;
	
	if (widget == mm1_gui::objects_submenu[0].widget) udata = SIGNAL_MM1_OBJECTS_RIBBON;
	if (widget == mm1_gui::objects_submenu[1].widget) udata = SIGNAL_MM1_OBJECTS_EPLANE;
	if (widget == mm1_gui::objects_submenu[2].widget) udata = SIGNAL_MM1_OBJECTS_ESURFACE;
	if (widget == mm1_gui::objects_submenu[3].widget) udata = SIGNAL_MM1_OBJECTS_VDWSURFACE;
	
	if (widget == mm1_gui::compute_submenu[0].widget) udata = SIGNAL_MM1_COMPUTE_GEOMOPT;
	if (widget == mm1_gui::compute_submenu[1].widget) udata = SIGNAL_MM1_COMPUTE_MOLDYN;
	
	if (widget == mm1_gui::popupmenu_info[10].widget) udata = SIGNAL_MM1_HYDROGENS;
	if (widget == mm1_gui::popupmenu_info[11].widget) udata = SIGNAL_MM1_BUILDER;
	
/*	if (widget == .widget) udata = ;
	if (widget == .widget) udata = ;
	if (widget == .widget) udata = ;
	if (widget == .widget) udata = ;
	if (widget == .widget) udata = ;
	if (widget == .widget) udata = ;
	if (widget == .widget) udata = ;
	if (widget == .widget) udata = ;	*/
	
	if (udata == NOT_DEFINED) cout << "unknown signal in GtkGLPopupHandler() !!!!!!!!" << endl;
	
////////////////////////////////////////
////////////////////////////////////////

	bs_gui * gui = win->gui;
	view_window * view_win = dynamic_cast<view_window *>(win);
	if (!view_win) { cout << "bad cast in GtkGLPopupHandler() !!!!!!!!" << endl; return; }
	
	char input;		// temporary...
	light * new_light;	// temporary...
	
	switch (udata)
	{
		case SIGNAL_GEN_VIEWS_NEWCAMERA:
		gui->AddCamera(view_win->cam->focus);
		break;
		
		case SIGNAL_GEN_VIEWS_NEWVIEW:
		gui->AddViewWindow(view_win->cam);
		break;
		
		case SIGNAL_GEN_VIEWS_DELETEVIEW:
//		gui->RemoveViewWindow(view_win);				// SOME PROBLEMS HERE !??!?!
		break;
		
		case SIGNAL_GEN_LIGHTS_NEWLIGHT:
		cout << "add light:" << endl;
		cout << "directional- or spotlight ? (d/s) "; cin >> input;
		
		if (input == 'd') new_light = new directional_light(ol_static());
		else if (input == 's')
		{
			new_light = new spot_light(ol_static());
			
			const fGL trans[3] = { 0.0, 0.0, -1.0 };
			new_light->TranslateObject((const fGL *) trans, view_win->cam->GetLocData());
		}
		else break;
		
		cout << "global or local light ? (g/l) "; cin >> input;
		
		if (input == 'g') gui->AddGlobalLight(new_light);
		else if (input == 'l') gui->AddLocalLight(new_light, view_win->cam);
		else delete new_light;
		
		if (input == 'g' || view_window::draw_info) gui->UpdateAllViewWindows();
		else if (input == 'l') gui->UpdateViewWindows(view_win->cam);
		break;
		
/*		case SIGNAL_POPUP_LIGHTS_SWITCH_LOCAL:
		win->cam->use_local_lights = !win->cam->use_local_lights;
		cout << "local lights = " << (win->cam->use_local_lights ? "on" : "off") << endl;
		gui->SetupLights(win->cam); gui->UpdateWindows(win->cam);
		break;
		
		case SIGNAL_POPUP_LIGHTS_SWITCH_GLOBAL:
		win->cam->use_global_lights = !win->cam->use_global_lights;
		cout << "global lights = " << (win->cam->use_global_lights ? "on" : "off") << endl;
		gui->SetupLights(win->cam); gui->UpdateWindows(win->cam);
		break;		*/
		
		default:
		gui->PopupEvent(view_win, udata);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

gtkgl_view_window::gtkgl_view_window(bs_gui * p1, camera * p2) : gtkgl_window(p1), view_window(p1, p2), window(p1)
{
}

gtkgl_view_window::~gtkgl_view_window(void)
{
}

void gtkgl_view_window::ExposeEvent(void)
{
	SetCurrent(); Render(); glFlush();
	gtk_gl_area_swap_buffers(GTK_GL_AREA(glarea));
}

////////////////////////////////////////////////////////////////////////////////////////////////////

gtkgl_factory * gtkgl_factory::instance = NULL;
singleton_cleaner<gtkgl_factory> gtkgl_factory_cleaner(gtkgl_factory::GetInstance());

gtkgl_factory::gtkgl_factory() : window_factory()
{
}

gtkgl_factory::~gtkgl_factory()
{
}

gtkgl_factory * gtkgl_factory::GetInstance(void)
{
	if (instance != NULL) return instance;
	else return (instance = new gtkgl_factory());
}

view_window * gtkgl_factory::ProduceView(bs_gui * mdl, camera * cam)
{
	gtkgl_view_window * win = new gtkgl_view_window(mdl, cam);
	win->CreateWindow(); return win;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
