// T3WINDOW.H : a view window class for GTK+/GNOME.

// Copyright (C) 1999 Tommi Hassinen, Ville Heikkil.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is for TARGET3...

#ifndef T3WINDOW_H
#define T3WINDOW_H

class gtkgl_window;
class gtkgl_view_window;

class gtkgl_factory;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include <gtk/gtk.h>
#include <gtkgl/gtkglarea.h>

#include "window.h"

#include <vector>

////////////////////////////////////////////////////////////////////////////////////////////////////

class gtkgl_window : virtual public window
{
	protected:
	
	GtkWidget * gtkwin;
	GtkWidget * glarea;	// this is the main widget!!!
	
	private:
	
	static vector<gtkgl_window *> gtkgl_vector;
	
	public:
	
	gtkgl_window(bs_gui *);
	virtual ~gtkgl_window(void);
	
	void CreateWindow(void);		// virtual
	void SetTitle(const char *);		// virtual
	void SetCurrent(void);			// virtual
	void Update(bool);			// virtual
	
	private:
	
	virtual void ExposeEvent(void) = 0;
	
	private:
	
	static gtkgl_window * GetWindow(GtkWidget *);
	
	static gint ExposeHandler(GtkWidget *, GdkEventExpose *);
	
	static gint ButtonHandler(GtkWidget *, GdkEventButton *);
	static gint MotionNotifyHandler(GtkWidget *, GdkEventMotion *);
	static gint ConfigureHandler(GtkWidget *, GdkEventConfigure *);
	static void DestroyHandler(GtkObject *);
	
	friend void GtkGLPopupHandler(GtkWidget *, void *);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class gtkgl_view_window : public gtkgl_window, public view_window
{
	public:
	
	gtkgl_view_window(bs_gui *, camera *);
	~gtkgl_view_window(void);
	
	private:
	
	void ExposeEvent(void);		// virtual
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class gtkgl_factory : public window_factory
{
	private:
	
	static gtkgl_factory * instance;
	gtkgl_factory(void);
	
	public:
	
	~gtkgl_factory(void);
	static gtkgl_factory * GetInstance(void);
	
	view_window * ProduceView(bs_gui *, camera *);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif  // T3WINDOW_H

// eof
